"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import s2clientprotocol.common_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ObservationFeatureLayer(google.protobuf.message.Message):
    """
    Observation - Feature Layer
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RENDERS_FIELD_NUMBER: builtins.int
    MINIMAP_RENDERS_FIELD_NUMBER: builtins.int
    @property
    def renders(self) -> global___FeatureLayers: ...
    @property
    def minimap_renders(self) -> global___FeatureLayersMinimap: ...
    def __init__(
        self,
        *,
        renders: global___FeatureLayers | None = ...,
        minimap_renders: global___FeatureLayersMinimap | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["minimap_renders", b"minimap_renders", "renders", b"renders"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["minimap_renders", b"minimap_renders", "renders", b"renders"]) -> None: ...

global___ObservationFeatureLayer = ObservationFeatureLayer

@typing_extensions.final
class FeatureLayers(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_MAP_FIELD_NUMBER: builtins.int
    VISIBILITY_MAP_FIELD_NUMBER: builtins.int
    CREEP_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    PLAYER_ID_FIELD_NUMBER: builtins.int
    UNIT_TYPE_FIELD_NUMBER: builtins.int
    SELECTED_FIELD_NUMBER: builtins.int
    UNIT_HIT_POINTS_FIELD_NUMBER: builtins.int
    UNIT_HIT_POINTS_RATIO_FIELD_NUMBER: builtins.int
    UNIT_ENERGY_FIELD_NUMBER: builtins.int
    UNIT_ENERGY_RATIO_FIELD_NUMBER: builtins.int
    UNIT_SHIELDS_FIELD_NUMBER: builtins.int
    UNIT_SHIELDS_RATIO_FIELD_NUMBER: builtins.int
    PLAYER_RELATIVE_FIELD_NUMBER: builtins.int
    UNIT_DENSITY_AA_FIELD_NUMBER: builtins.int
    UNIT_DENSITY_FIELD_NUMBER: builtins.int
    EFFECTS_FIELD_NUMBER: builtins.int
    HALLUCINATIONS_FIELD_NUMBER: builtins.int
    CLOAKED_FIELD_NUMBER: builtins.int
    BLIP_FIELD_NUMBER: builtins.int
    BUFFS_FIELD_NUMBER: builtins.int
    BUFF_DURATION_FIELD_NUMBER: builtins.int
    ACTIVE_FIELD_NUMBER: builtins.int
    BUILD_PROGRESS_FIELD_NUMBER: builtins.int
    BUILDABLE_FIELD_NUMBER: builtins.int
    PATHABLE_FIELD_NUMBER: builtins.int
    PLACEHOLDER_FIELD_NUMBER: builtins.int
    @property
    def height_map(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Terrain height. World space units of [-200, 200] encoded into [0, 255]."""
    @property
    def visibility_map(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. 0=Hidden, 1=Fogged, 2=Visible, 3=FullHidden"""
    @property
    def creep(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Zerg creep."""
    @property
    def power(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Protoss power."""
    @property
    def player_id(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Participants: [1, 15] Neutral: 16"""
    @property
    def unit_type(self) -> s2clientprotocol.common_pb2.ImageData:
        """int32. Unique identifier for type of unit."""
    @property
    def selected(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Selected units."""
    @property
    def unit_hit_points(self) -> s2clientprotocol.common_pb2.ImageData:
        """int32."""
    @property
    def unit_hit_points_ratio(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Ratio of current health to max health. [0%, 100%] encoded into [0, 255]."""
    @property
    def unit_energy(self) -> s2clientprotocol.common_pb2.ImageData:
        """int32."""
    @property
    def unit_energy_ratio(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Ratio of current energy to max energy. [0%, 100%] encoded into [0, 255]."""
    @property
    def unit_shields(self) -> s2clientprotocol.common_pb2.ImageData:
        """int32."""
    @property
    def unit_shields_ratio(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Ratio of current shields to max shields. [0%, 100%] encoded into [0, 255]."""
    @property
    def player_relative(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. See "Alliance" enum in raw.proto. Range: [1, 4]"""
    @property
    def unit_density_aa(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Density of units overlapping a pixel, anti-aliased. [0.0, 16.0f] encoded into [0, 255]."""
    @property
    def unit_density(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Count of units overlapping a pixel."""
    @property
    def effects(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Visuals of persistent abilities. (eg. Psistorm)"""
    @property
    def hallucinations(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether the unit here is a hallucination."""
    @property
    def cloaked(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether the unit here is cloaked. Hidden units will show up too, but with less details in other layers."""
    @property
    def blip(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether the unit here is a blip."""
    @property
    def buffs(self) -> s2clientprotocol.common_pb2.ImageData:
        """int32. One of the buffs applied to this unit. Extras are ignored."""
    @property
    def buff_duration(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Ratio of buff remaining. [0%, 100%] encoded into [0, 255]."""
    @property
    def active(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether the unit here is active."""
    @property
    def build_progress(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. How far along the building is building something. [0%, 100%] encoded into [0, 255]."""
    @property
    def buildable(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether a building can be built here."""
    @property
    def pathable(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether a unit can walk here."""
    @property
    def placeholder(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether the unit here is a placeholder building to be constructed."""
    def __init__(
        self,
        *,
        height_map: s2clientprotocol.common_pb2.ImageData | None = ...,
        visibility_map: s2clientprotocol.common_pb2.ImageData | None = ...,
        creep: s2clientprotocol.common_pb2.ImageData | None = ...,
        power: s2clientprotocol.common_pb2.ImageData | None = ...,
        player_id: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_type: s2clientprotocol.common_pb2.ImageData | None = ...,
        selected: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_hit_points: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_hit_points_ratio: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_energy: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_energy_ratio: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_shields: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_shields_ratio: s2clientprotocol.common_pb2.ImageData | None = ...,
        player_relative: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_density_aa: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_density: s2clientprotocol.common_pb2.ImageData | None = ...,
        effects: s2clientprotocol.common_pb2.ImageData | None = ...,
        hallucinations: s2clientprotocol.common_pb2.ImageData | None = ...,
        cloaked: s2clientprotocol.common_pb2.ImageData | None = ...,
        blip: s2clientprotocol.common_pb2.ImageData | None = ...,
        buffs: s2clientprotocol.common_pb2.ImageData | None = ...,
        buff_duration: s2clientprotocol.common_pb2.ImageData | None = ...,
        active: s2clientprotocol.common_pb2.ImageData | None = ...,
        build_progress: s2clientprotocol.common_pb2.ImageData | None = ...,
        buildable: s2clientprotocol.common_pb2.ImageData | None = ...,
        pathable: s2clientprotocol.common_pb2.ImageData | None = ...,
        placeholder: s2clientprotocol.common_pb2.ImageData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["active", b"active", "blip", b"blip", "buff_duration", b"buff_duration", "buffs", b"buffs", "build_progress", b"build_progress", "buildable", b"buildable", "cloaked", b"cloaked", "creep", b"creep", "effects", b"effects", "hallucinations", b"hallucinations", "height_map", b"height_map", "pathable", b"pathable", "placeholder", b"placeholder", "player_id", b"player_id", "player_relative", b"player_relative", "power", b"power", "selected", b"selected", "unit_density", b"unit_density", "unit_density_aa", b"unit_density_aa", "unit_energy", b"unit_energy", "unit_energy_ratio", b"unit_energy_ratio", "unit_hit_points", b"unit_hit_points", "unit_hit_points_ratio", b"unit_hit_points_ratio", "unit_shields", b"unit_shields", "unit_shields_ratio", b"unit_shields_ratio", "unit_type", b"unit_type", "visibility_map", b"visibility_map"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["active", b"active", "blip", b"blip", "buff_duration", b"buff_duration", "buffs", b"buffs", "build_progress", b"build_progress", "buildable", b"buildable", "cloaked", b"cloaked", "creep", b"creep", "effects", b"effects", "hallucinations", b"hallucinations", "height_map", b"height_map", "pathable", b"pathable", "placeholder", b"placeholder", "player_id", b"player_id", "player_relative", b"player_relative", "power", b"power", "selected", b"selected", "unit_density", b"unit_density", "unit_density_aa", b"unit_density_aa", "unit_energy", b"unit_energy", "unit_energy_ratio", b"unit_energy_ratio", "unit_hit_points", b"unit_hit_points", "unit_hit_points_ratio", b"unit_hit_points_ratio", "unit_shields", b"unit_shields", "unit_shields_ratio", b"unit_shields_ratio", "unit_type", b"unit_type", "visibility_map", b"visibility_map"]) -> None: ...

global___FeatureLayers = FeatureLayers

@typing_extensions.final
class FeatureLayersMinimap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_MAP_FIELD_NUMBER: builtins.int
    VISIBILITY_MAP_FIELD_NUMBER: builtins.int
    CREEP_FIELD_NUMBER: builtins.int
    CAMERA_FIELD_NUMBER: builtins.int
    PLAYER_ID_FIELD_NUMBER: builtins.int
    PLAYER_RELATIVE_FIELD_NUMBER: builtins.int
    SELECTED_FIELD_NUMBER: builtins.int
    ALERTS_FIELD_NUMBER: builtins.int
    BUILDABLE_FIELD_NUMBER: builtins.int
    PATHABLE_FIELD_NUMBER: builtins.int
    UNIT_TYPE_FIELD_NUMBER: builtins.int
    @property
    def height_map(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Terrain height. World space units of [-200, 200] encoded into [0, 255]."""
    @property
    def visibility_map(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. 0=Hidden, 1=Fogged, 2=Visible, 3=FullHidden"""
    @property
    def creep(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Zerg creep."""
    @property
    def camera(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Area covered by the camera."""
    @property
    def player_id(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. Participants: [1, 15] Neutral: 16"""
    @property
    def player_relative(self) -> s2clientprotocol.common_pb2.ImageData:
        """uint8. See "Alliance" enum in raw.proto. Range: [1, 4]"""
    @property
    def selected(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Selected units."""
    @property
    def alerts(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Shows 'UnitAttacked' alert location."""
    @property
    def buildable(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether a building can be built here."""
    @property
    def pathable(self) -> s2clientprotocol.common_pb2.ImageData:
        """1-bit. Whether a unit can walk here."""
    @property
    def unit_type(self) -> s2clientprotocol.common_pb2.ImageData:
        """Cheat layers, enable with SpatialCameraSetup.allow_cheating_layers.
        int32. Unique identifier for type of unit.
        """
    def __init__(
        self,
        *,
        height_map: s2clientprotocol.common_pb2.ImageData | None = ...,
        visibility_map: s2clientprotocol.common_pb2.ImageData | None = ...,
        creep: s2clientprotocol.common_pb2.ImageData | None = ...,
        camera: s2clientprotocol.common_pb2.ImageData | None = ...,
        player_id: s2clientprotocol.common_pb2.ImageData | None = ...,
        player_relative: s2clientprotocol.common_pb2.ImageData | None = ...,
        selected: s2clientprotocol.common_pb2.ImageData | None = ...,
        alerts: s2clientprotocol.common_pb2.ImageData | None = ...,
        buildable: s2clientprotocol.common_pb2.ImageData | None = ...,
        pathable: s2clientprotocol.common_pb2.ImageData | None = ...,
        unit_type: s2clientprotocol.common_pb2.ImageData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["alerts", b"alerts", "buildable", b"buildable", "camera", b"camera", "creep", b"creep", "height_map", b"height_map", "pathable", b"pathable", "player_id", b"player_id", "player_relative", b"player_relative", "selected", b"selected", "unit_type", b"unit_type", "visibility_map", b"visibility_map"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["alerts", b"alerts", "buildable", b"buildable", "camera", b"camera", "creep", b"creep", "height_map", b"height_map", "pathable", b"pathable", "player_id", b"player_id", "player_relative", b"player_relative", "selected", b"selected", "unit_type", b"unit_type", "visibility_map", b"visibility_map"]) -> None: ...

global___FeatureLayersMinimap = FeatureLayersMinimap

@typing_extensions.final
class ObservationRender(google.protobuf.message.Message):
    """
    Observation - Rendered
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAP_FIELD_NUMBER: builtins.int
    MINIMAP_FIELD_NUMBER: builtins.int
    @property
    def map(self) -> s2clientprotocol.common_pb2.ImageData: ...
    @property
    def minimap(self) -> s2clientprotocol.common_pb2.ImageData: ...
    def __init__(
        self,
        *,
        map: s2clientprotocol.common_pb2.ImageData | None = ...,
        minimap: s2clientprotocol.common_pb2.ImageData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["map", b"map", "minimap", b"minimap"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["map", b"map", "minimap", b"minimap"]) -> None: ...

global___ObservationRender = ObservationRender

@typing_extensions.final
class ActionSpatial(google.protobuf.message.Message):
    """
    Action
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_COMMAND_FIELD_NUMBER: builtins.int
    CAMERA_MOVE_FIELD_NUMBER: builtins.int
    UNIT_SELECTION_POINT_FIELD_NUMBER: builtins.int
    UNIT_SELECTION_RECT_FIELD_NUMBER: builtins.int
    @property
    def unit_command(self) -> global___ActionSpatialUnitCommand: ...
    @property
    def camera_move(self) -> global___ActionSpatialCameraMove: ...
    @property
    def unit_selection_point(self) -> global___ActionSpatialUnitSelectionPoint: ...
    @property
    def unit_selection_rect(self) -> global___ActionSpatialUnitSelectionRect: ...
    def __init__(
        self,
        *,
        unit_command: global___ActionSpatialUnitCommand | None = ...,
        camera_move: global___ActionSpatialCameraMove | None = ...,
        unit_selection_point: global___ActionSpatialUnitSelectionPoint | None = ...,
        unit_selection_rect: global___ActionSpatialUnitSelectionRect | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["action", b"action", "camera_move", b"camera_move", "unit_command", b"unit_command", "unit_selection_point", b"unit_selection_point", "unit_selection_rect", b"unit_selection_rect"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "camera_move", b"camera_move", "unit_command", b"unit_command", "unit_selection_point", b"unit_selection_point", "unit_selection_rect", b"unit_selection_rect"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["action", b"action"]) -> typing_extensions.Literal["unit_command", "camera_move", "unit_selection_point", "unit_selection_rect"] | None: ...

global___ActionSpatial = ActionSpatial

@typing_extensions.final
class ActionSpatialUnitCommand(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    TARGET_SCREEN_COORD_FIELD_NUMBER: builtins.int
    TARGET_MINIMAP_COORD_FIELD_NUMBER: builtins.int
    QUEUE_COMMAND_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    @property
    def target_screen_coord(self) -> s2clientprotocol.common_pb2.PointI: ...
    @property
    def target_minimap_coord(self) -> s2clientprotocol.common_pb2.PointI: ...
    queue_command: builtins.bool
    """Equivalent to shift+command."""
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        target_screen_coord: s2clientprotocol.common_pb2.PointI | None = ...,
        target_minimap_coord: s2clientprotocol.common_pb2.PointI | None = ...,
        queue_command: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "queue_command", b"queue_command", "target", b"target", "target_minimap_coord", b"target_minimap_coord", "target_screen_coord", b"target_screen_coord"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "queue_command", b"queue_command", "target", b"target", "target_minimap_coord", b"target_minimap_coord", "target_screen_coord", b"target_screen_coord"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["target", b"target"]) -> typing_extensions.Literal["target_screen_coord", "target_minimap_coord"] | None: ...

global___ActionSpatialUnitCommand = ActionSpatialUnitCommand

@typing_extensions.final
class ActionSpatialCameraMove(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CENTER_MINIMAP_FIELD_NUMBER: builtins.int
    @property
    def center_minimap(self) -> s2clientprotocol.common_pb2.PointI:
        """Simulates a click on the minimap to move the camera."""
    def __init__(
        self,
        *,
        center_minimap: s2clientprotocol.common_pb2.PointI | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["center_minimap", b"center_minimap"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["center_minimap", b"center_minimap"]) -> None: ...

global___ActionSpatialCameraMove = ActionSpatialCameraMove

@typing_extensions.final
class ActionSpatialUnitSelectionPoint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionSpatialUnitSelectionPoint._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Select: ActionSpatialUnitSelectionPoint._Type.ValueType  # 1
        """Equivalent to normal click. Changes selection to unit."""
        Toggle: ActionSpatialUnitSelectionPoint._Type.ValueType  # 2
        """Equivalent to shift+click. Toggle selection of unit."""
        AllType: ActionSpatialUnitSelectionPoint._Type.ValueType  # 3
        """Equivalent to control+click. Selects all units of a given type."""
        AddAllType: ActionSpatialUnitSelectionPoint._Type.ValueType  # 4
        """Equivalent to shift+control+click. Selects all units of a given type."""

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    Select: ActionSpatialUnitSelectionPoint.Type.ValueType  # 1
    """Equivalent to normal click. Changes selection to unit."""
    Toggle: ActionSpatialUnitSelectionPoint.Type.ValueType  # 2
    """Equivalent to shift+click. Toggle selection of unit."""
    AllType: ActionSpatialUnitSelectionPoint.Type.ValueType  # 3
    """Equivalent to control+click. Selects all units of a given type."""
    AddAllType: ActionSpatialUnitSelectionPoint.Type.ValueType  # 4
    """Equivalent to shift+control+click. Selects all units of a given type."""

    SELECTION_SCREEN_COORD_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    @property
    def selection_screen_coord(self) -> s2clientprotocol.common_pb2.PointI: ...
    type: global___ActionSpatialUnitSelectionPoint.Type.ValueType
    def __init__(
        self,
        *,
        selection_screen_coord: s2clientprotocol.common_pb2.PointI | None = ...,
        type: global___ActionSpatialUnitSelectionPoint.Type.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["selection_screen_coord", b"selection_screen_coord", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["selection_screen_coord", b"selection_screen_coord", "type", b"type"]) -> None: ...

global___ActionSpatialUnitSelectionPoint = ActionSpatialUnitSelectionPoint

@typing_extensions.final
class ActionSpatialUnitSelectionRect(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SELECTION_SCREEN_COORD_FIELD_NUMBER: builtins.int
    SELECTION_ADD_FIELD_NUMBER: builtins.int
    @property
    def selection_screen_coord(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[s2clientprotocol.common_pb2.RectangleI]:
        """Eventually this should not be an array, but a single field (multiple would be cheating)."""
    selection_add: builtins.bool
    """Equivalent to shift+drag. Adds units to selection."""
    def __init__(
        self,
        *,
        selection_screen_coord: collections.abc.Iterable[s2clientprotocol.common_pb2.RectangleI] | None = ...,
        selection_add: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["selection_add", b"selection_add", "selection_screen_coord", b"selection_screen_coord"]) -> None: ...

global___ActionSpatialUnitSelectionRect = ActionSpatialUnitSelectionRect



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def dispersion(
    G,
    u: Incomplete | None = None,
    v: Incomplete | None = None,
    normalized: bool = True,
    alpha: float = 1.0,
    b: float = 0.0,
    c: float = 0.0,
): ...



from typing import Literal

AF_INET: Literal[2]
AF_INET6: Literal[10]

def inet_ntoa(packed_ip: bytes) -> str: ...
def inet_ntop(af: int, packed_ip: bytes) -> str: ...
def inet_pton(af: int, ip_string: str) -> str: ...



from typing import Final

XK_Hstroke: Final = 0x2A1
XK_Hcircumflex: Final = 0x2A6
XK_Iabovedot: Final = 0x2A9
XK_Gbreve: Final = 0x2AB
XK_Jcircumflex: Final = 0x2AC
XK_hstroke: Final = 0x2B1
XK_hcircumflex: Final = 0x2B6
XK_idotless: Final = 0x2B9
XK_gbreve: Final = 0x2BB
XK_jcircumflex: Final = 0x2BC
XK_Cabovedot: Final = 0x2C5
XK_Ccircumflex: Final = 0x2C6
XK_Gabovedot: Final = 0x2D5
XK_Gcircumflex: Final = 0x2D8
XK_Ubreve: Final = 0x2DD
XK_Scircumflex: Final = 0x2DE
XK_cabovedot: Final = 0x2E5
XK_ccircumflex: Final = 0x2E6
XK_gabovedot: Final = 0x2F5
XK_gcircumflex: Final = 0x2F8
XK_ubreve: Final = 0x2FD
XK_scircumflex: Final = 0x2FE



from collections.abc import Iterable

from fanstatic.core import Bundle, NeededResources, Resource

def bundle_resources(resources: Iterable[Resource]) -> list[Resource | Bundle]: ...
def rollup_resources(resources: Iterable[Resource]) -> set[Resource]: ...
def sort_resources(resources: Iterable[Resource]) -> list[Resource]: ...

class Inclusion:
    needed: NeededResources
    resources: list[Resource | Bundle]
    def __init__(
        self,
        needed: NeededResources,
        resources: Iterable[Resource] | None = None,
        compile: bool = False,
        bundle: bool = False,
        mode: str | None = None,
        rollup: bool = False,
    ) -> None: ...
    def __len__(self) -> int: ...
    def render(self) -> str: ...



import abc
from _typeshed import Incomplete
from abc import abstractmethod

text_type = str
binary_type = bytes
READ_CHUNK_SIZE: int
EINVAL: Incomplete

class SpooledIOBase(metaclass=abc.ABCMeta):
    __metaclass__: Incomplete
    def __init__(self, max_size: int = 5000000, dir: Incomplete | None = None) -> None: ...
    @abstractmethod
    def read(self, n: int = -1): ...
    @abstractmethod
    def write(self, s): ...
    @abstractmethod
    def seek(self, pos, mode: int = 0): ...
    @abstractmethod
    def readline(self, length: Incomplete | None = None): ...
    @abstractmethod
    def readlines(self, sizehint: int = 0): ...
    def writelines(self, lines) -> None: ...
    @abstractmethod
    def rollover(self): ...
    @abstractmethod
    def tell(self): ...
    @property
    @abc.abstractmethod
    def buffer(self): ...
    @property
    @abc.abstractmethod
    def len(self): ...
    softspace: Incomplete
    def close(self): ...
    def flush(self): ...
    def isatty(self): ...
    def next(self): ...
    @property
    def closed(self): ...
    @property
    def pos(self): ...
    @property
    def buf(self): ...
    def fileno(self): ...
    def truncate(self, size: Incomplete | None = None): ...
    def getvalue(self): ...
    def seekable(self): ...
    def readable(self): ...
    def writable(self): ...
    __next__: Incomplete
    def __len__(self): ...
    def __iter__(self): ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __bool__(self): ...
    def __del__(self) -> None: ...
    __nonzero__: Incomplete

class SpooledBytesIO(SpooledIOBase):
    def read(self, n: int = -1): ...
    def write(self, s) -> None: ...
    def seek(self, pos, mode: int = 0): ...
    def readline(self, length: Incomplete | None = None): ...
    def readlines(self, sizehint: int = 0): ...
    def rollover(self) -> None: ...
    @property
    def buffer(self): ...
    @property
    def len(self): ...
    def tell(self): ...

class SpooledStringIO(SpooledIOBase):
    def __init__(self, *args, **kwargs) -> None: ...
    def read(self, n: int = -1): ...
    def write(self, s) -> None: ...
    def seek(self, pos, mode: int = 0): ...
    def readline(self, length: Incomplete | None = None): ...
    def readlines(self, sizehint: int = 0): ...
    @property
    def buffer(self): ...
    def rollover(self) -> None: ...
    def tell(self): ...
    @property
    def len(self): ...

def is_text_fileobj(fileobj) -> bool: ...

class MultiFileReader:
    def __init__(self, *fileobjs) -> None: ...
    def read(self, amt: Incomplete | None = None): ...
    def seek(self, offset, whence=0) -> None: ...



from _typeshed import Incomplete

from ...extend.operation import ExtendedOperation
from ...protocol.novell import NmasGetUniversalPasswordRequestValue, NmasGetUniversalPasswordResponseValue

class NmasGetUniversalPassword(ExtendedOperation):
    request_name: str
    response_name: str
    request_value: NmasGetUniversalPasswordRequestValue
    asn1_spec: NmasGetUniversalPasswordResponseValue
    response_attribute: str
    def config(self) -> None: ...
    def __init__(self, connection, user, controls: Incomplete | None = None) -> None: ...
    def populate_result(self) -> None: ...



from .pil import (
    CircleModuleDrawer as CircleModuleDrawer,
    GappedSquareModuleDrawer as GappedSquareModuleDrawer,
    HorizontalBarsDrawer as HorizontalBarsDrawer,
    RoundedModuleDrawer as RoundedModuleDrawer,
    SquareModuleDrawer as SquareModuleDrawer,
    VerticalBarsDrawer as VerticalBarsDrawer,
)



from typing import ClassVar

from .core import UnitedStates

class Michigan(UnitedStates):
    include_christmas_eve: ClassVar[bool]
    include_thanksgiving_friday: ClassVar[bool]
    include_election_day_even: ClassVar[bool]
    include_columbus_day: ClassVar[bool]
    def get_fixed_holidays(self, year): ...



import io
from _typeshed import ReadableBuffer, WriteableBuffer
from abc import abstractmethod
from collections.abc import Callable, Generator
from typing import Any, Final

from serial.rs485 import RS485Settings

XON: Final = b"\x11"
XOFF: Final = b"\x13"
CR: Final = b"\r"
LF: Final = b"\n"
PARITY_NONE: Final = "N"
PARITY_EVEN: Final = "E"
PARITY_ODD: Final = "O"
PARITY_MARK: Final = "M"
PARITY_SPACE: Final = "S"
STOPBITS_ONE: Final = 1
STOPBITS_ONE_POINT_FIVE: float
STOPBITS_TWO: Final = 2
FIVEBITS: Final = 5
SIXBITS: Final = 6
SEVENBITS: Final = 7
EIGHTBITS: Final = 8
PARITY_NAMES: dict[str, str]

class SerialException(OSError): ...
class SerialTimeoutException(SerialException): ...

class PortNotOpenError(SerialException):
    def __init__(self) -> None: ...

class Timeout:
    TIME: Callable[[], float]
    is_infinite: bool
    is_non_blocking: bool
    duration: float
    target_time: float
    def __init__(self, duration: float) -> None: ...
    def expired(self) -> bool: ...
    def time_left(self) -> float: ...
    def restart(self, duration: float) -> None: ...

class SerialBase(io.RawIOBase):
    BAUDRATES: tuple[int, ...]
    BYTESIZES: tuple[int, ...]
    PARITIES: tuple[str, ...]
    STOPBITS: tuple[int, float, int]
    is_open: bool
    portstr: str | None
    name: str | None
    def __init__(
        self,
        port: str | None = None,
        baudrate: int = 9600,
        bytesize: int = 8,
        parity: str = "N",
        stopbits: float = 1,
        timeout: float | None = None,
        xonxoff: bool = False,
        rtscts: bool = False,
        write_timeout: float | None = None,
        dsrdtr: bool = False,
        inter_byte_timeout: float | None = None,
        exclusive: float | None = None,
    ) -> None: ...

    # Return type:
    # ------------
    # `io.RawIOBase`, the super class, declares the return type of read as `-> bytes | None`.
    # `SerialBase` does not define `read` at runtime but REQUIRES subclasses to implement it and
    # require it to return `bytes`.
    # Abstract:
    # ---------
    # `io.RawIOBase` implements `read` in terms of `readinto`. `SerialBase` implements `readinto`
    # in terms of `read`. If subclasses do not implement `read`, any call to `read` or `read_into`
    # will fail at runtime with a `RecursionError`.
    @abstractmethod
    def read(self, size: int = -1, /) -> bytes: ...
    @abstractmethod
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    @property
    def port(self) -> str | None: ...
    @port.setter
    def port(self, port: str | None) -> None: ...
    @property
    def baudrate(self) -> int: ...
    @baudrate.setter
    def baudrate(self, baudrate: int) -> None: ...
    @property
    def bytesize(self) -> int: ...
    @bytesize.setter
    def bytesize(self, bytesize: int) -> None: ...
    @property
    def exclusive(self) -> bool | None: ...
    @exclusive.setter
    def exclusive(self, exclusive: bool | None) -> None: ...
    @property
    def parity(self) -> str: ...
    @parity.setter
    def parity(self, parity: str) -> None: ...
    @property
    def stopbits(self) -> float: ...
    @stopbits.setter
    def stopbits(self, stopbits: float) -> None: ...
    @property
    def timeout(self) -> float | None: ...
    @timeout.setter
    def timeout(self, timeout: float | None) -> None: ...
    @property
    def write_timeout(self) -> float | None: ...
    @write_timeout.setter
    def write_timeout(self, timeout: float | None) -> None: ...
    @property
    def inter_byte_timeout(self) -> float | None: ...
    @inter_byte_timeout.setter
    def inter_byte_timeout(self, ic_timeout: float | None) -> None: ...
    @property
    def xonxoff(self) -> bool: ...
    @xonxoff.setter
    def xonxoff(self, xonxoff: bool) -> None: ...
    @property
    def rtscts(self) -> bool: ...
    @rtscts.setter
    def rtscts(self, rtscts: bool) -> None: ...
    @property
    def dsrdtr(self) -> bool: ...
    @dsrdtr.setter
    def dsrdtr(self, dsrdtr: bool | None = ...) -> None: ...
    @property
    def rts(self) -> bool: ...
    @rts.setter
    def rts(self, value: bool) -> None: ...
    @property
    def dtr(self) -> bool: ...
    @dtr.setter
    def dtr(self, value: bool) -> None: ...
    @property
    def break_condition(self) -> bool: ...
    @break_condition.setter
    def break_condition(self, value: bool) -> None: ...
    @property
    def rs485_mode(self) -> RS485Settings | None: ...
    @rs485_mode.setter
    def rs485_mode(self, rs485_settings: RS485Settings | None) -> None: ...
    def get_settings(self) -> dict[str, Any]: ...
    def apply_settings(self, d: dict[str, Any]) -> None: ...
    def readinto(self, buffer: WriteableBuffer, /) -> int: ...  # returns int unlike `io.RawIOBase`
    def send_break(self, duration: float = 0.25) -> None: ...
    def read_all(self) -> bytes | None: ...
    def read_until(self, expected: bytes = b"\n", size: int | None = None) -> bytes: ...
    def iread_until(self, expected: bytes = ..., size: int | None = ...) -> Generator[bytes, None, None]: ...



from _typeshed import Incomplete

class AXEngine:
    eScript: Incomplete
    eParse: Incomplete
    eSafety: Incomplete
    def __init__(self, site, engine) -> None: ...
    def __del__(self) -> None: ...
    def GetScriptDispatch(self, name: Incomplete | None = ...): ...
    def AddNamedItem(self, item, flags): ...
    def AddCode(self, code, flags: int = ...) -> None: ...
    def EvalCode(self, code): ...
    def Start(self) -> None: ...
    def Close(self) -> None: ...
    def SetScriptState(self, state) -> None: ...

IActiveScriptSite_methods: Incomplete

class AXSite:
    lcid: Incomplete
    objModel: Incomplete
    engine: Incomplete
    def __init__(self, objModel=..., engine: Incomplete | None = ..., lcid: int = ...) -> None: ...
    def AddEngine(self, engine): ...
    def GetLCID(self): ...
    def GetItemInfo(self, name, returnMask): ...
    def GetDocVersionString(self): ...
    def OnScriptTerminate(self, result, excepInfo) -> None: ...
    def OnStateChange(self, state) -> None: ...
    def OnScriptError(self, errorInterface): ...
    def OnEnterScript(self) -> None: ...
    def OnLeaveScript(self) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def harmonic_centrality(
    G, nbunch: Incomplete | None = None, distance: Incomplete | None = None, sources: Incomplete | None = None
): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def dominating_set(G, start_with: Incomplete | None = None): ...
@_dispatch
def is_dominating_set(G, nbunch): ...



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Capabilities(SystemBackendMixin):
    def get_capabilities(self, paths, token: Incomplete | None = None, accessor: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import Final

from braintree.resource import Resource
from braintree.subscription import Subscription

class ApplePayCard(Resource):
    class CardType:
        AmEx: Final = "Apple Pay - American Express"
        MasterCard: Final = "Apple Pay - MasterCard"
        Visa: Final = "Apple Pay - Visa"

    is_expired: Incomplete
    subscriptions: list[Subscription]
    def __init__(self, gateway, attributes) -> None: ...
    @property
    def expiration_date(self): ...
    @staticmethod
    def signature(): ...



import ast
import sys
from collections.abc import Callable, Iterable, Iterator
from contextlib import contextmanager
from re import Pattern
from typing import Any, ClassVar, Literal, TypeVar, overload
from typing_extensions import ParamSpec, TypeAlias

from pyflakes.messages import Message

_AnyFunction: TypeAlias = Callable[..., Any]
_F = TypeVar("_F", bound=_AnyFunction)
_P = ParamSpec("_P")
_T = TypeVar("_T")

PYPY: bool

def getAlternatives(n: ast.If | ast.Try) -> list[ast.AST]: ...

FOR_TYPES: tuple[type[ast.For], type[ast.AsyncFor]]
MAPPING_KEY_RE: Pattern[str]
CONVERSION_FLAG_RE: Pattern[str]
WIDTH_RE: Pattern[str]
PRECISION_RE: Pattern[str]
LENGTH_RE: Pattern[str]
VALID_CONVERSIONS: frozenset[str]

_FormatType: TypeAlias = tuple[str | None, str | None, str | None, str | None, str]
_PercentFormat: TypeAlias = tuple[str, _FormatType | None]

def parse_percent_format(s: str) -> tuple[_PercentFormat, ...]: ...

class _FieldsOrder(dict[type[ast.AST], tuple[str, ...]]):
    def __missing__(self, node_class: type[ast.AST]) -> tuple[str, ...]: ...

def counter(items: Iterable[_T]) -> dict[_T, int]: ...

_OmitType: TypeAlias = str | tuple[str, ...] | None

def iter_child_nodes(node: ast.AST, omit: _OmitType = None, _fields_order: _FieldsOrder = ...) -> Iterator[ast.AST]: ...
@overload
def convert_to_value(item: ast.Str) -> str: ...  # type: ignore[overload-overlap]
@overload
def convert_to_value(item: ast.Bytes) -> bytes: ...  # type: ignore[overload-overlap]
@overload
def convert_to_value(item: ast.Tuple) -> tuple[Any, ...]: ...  # type: ignore[overload-overlap]
@overload
def convert_to_value(item: ast.Name | ast.NameConstant) -> Any: ...
@overload
def convert_to_value(item: ast.AST) -> UnhandledKeyType: ...
def is_notimplemented_name_node(node: object) -> bool: ...

class Binding:
    name: str
    source: ast.AST | None
    used: Literal[False] | tuple[Any, ast.AST]
    def __init__(self, name: str, source: ast.AST | None) -> None: ...
    def redefines(self, other: Binding) -> bool: ...

class Definition(Binding): ...

class Builtin(Definition):
    def __init__(self, name: str) -> None: ...

class UnhandledKeyType: ...

class VariableKey:
    name: str
    def __init__(self, item: ast.Name) -> None: ...
    def __eq__(self, compare: object) -> bool: ...
    def __hash__(self) -> int: ...

class Importation(Definition):
    fullName: str
    redefined: list[Any]
    def __init__(self, name: str, source: ast.AST | None, full_name: str | None = None) -> None: ...
    @property
    def source_statement(self) -> str: ...

class SubmoduleImportation(Importation):
    def __init__(self, name: str, source: ast.Import | None) -> None: ...

class ImportationFrom(Importation):
    module: str
    real_name: str
    def __init__(self, name: str, source: ast.AST, module: str, real_name: str | None = None) -> None: ...

class StarImportation(Importation):
    def __init__(self, name: str, source: ast.AST) -> None: ...

class FutureImportation(ImportationFrom):
    used: tuple[Any, ast.AST]
    def __init__(self, name: str, source: ast.AST, scope) -> None: ...

class Argument(Binding): ...
class Assignment(Binding): ...

class Annotation(Binding):
    def redefines(self, other: Binding) -> Literal[False]: ...

class FunctionDefinition(Definition): ...
class ClassDefinition(Definition): ...

class ExportBinding(Binding):
    names: list[str]
    def __init__(self, name: str, source: ast.AST, scope: Scope) -> None: ...

class Scope(dict[str, Binding]):
    importStarred: bool

class ClassScope(Scope): ...

class FunctionScope(Scope):
    usesLocals: bool
    alwaysUsed: ClassVar[set[str]]
    globals: set[str]
    returnValue: Any
    isGenerator: bool
    def __init__(self) -> None: ...
    def unused_assignments(self) -> Iterator[tuple[str, Binding]]: ...
    def unused_annotations(self) -> Iterator[tuple[str, Annotation]]: ...

class TypeScope(Scope): ...
class GeneratorScope(Scope): ...
class ModuleScope(Scope): ...
class DoctestScope(ModuleScope): ...

class DetectClassScopedMagic:
    names: list[str]

def getNodeName(node: ast.AST) -> str: ...

TYPING_MODULES: frozenset[Literal["typing", "typing_extensions"]]

def is_typing_overload(value: Binding, scope_stack) -> bool: ...

class AnnotationState:
    NONE: ClassVar[Literal[0]]
    STRING: ClassVar[Literal[1]]
    BARE: ClassVar[Literal[2]]

def in_annotation(func: _F) -> _F: ...
def in_string_annotation(func: _F) -> _F: ...

if sys.version_info >= (3, 10):
    _Match: TypeAlias = ast.Match
    _MatchCase: TypeAlias = ast.match_case
    _MatchValue: TypeAlias = ast.MatchValue
    _MatchSingleton: TypeAlias = ast.MatchSingleton
    _MatchSequence: TypeAlias = ast.MatchSequence
    _MatchStar: TypeAlias = ast.MatchStar
    _MatchMapping: TypeAlias = ast.MatchMapping
    _MatchClass: TypeAlias = ast.MatchClass
    _MatchAs: TypeAlias = ast.MatchAs
    _MatchOr: TypeAlias = ast.MatchOr
else:
    _Match: TypeAlias = Any
    _MatchCase: TypeAlias = Any
    _MatchValue: TypeAlias = Any
    _MatchSingleton: TypeAlias = Any
    _MatchSequence: TypeAlias = Any
    _MatchStar: TypeAlias = Any
    _MatchMapping: TypeAlias = Any
    _MatchClass: TypeAlias = Any
    _MatchAs: TypeAlias = Any
    _MatchOr: TypeAlias = Any

if sys.version_info >= (3, 12):
    _TypeVar: TypeAlias = ast.TypeVar
    _TypeAlias: TypeAlias = ast.TypeAlias
else:
    _TypeVar: TypeAlias = Any
    _TypeAlias: TypeAlias = Any

class Checker:
    nodeDepth: int
    offset: tuple[int, int] | None
    builtIns: set[str]
    deadScopes: list[Any]
    messages: list[Any]
    filename: str
    withDoctest: bool
    scopeStack: list[Scope]
    exceptHandlers: list[Any]
    root: ast.AST
    def __init__(
        self,
        tree: ast.AST,
        filename: str = "(none)",
        builtins: Iterable[str] | None = None,
        withDoctest: bool = False,
        file_tokens: tuple[Any, ...] = (),
    ) -> None: ...
    def deferFunction(self, callable: _AnyFunction) -> None: ...
    @property
    def futuresAllowed(self) -> bool: ...
    @futuresAllowed.setter
    def futuresAllowed(self, value: Literal[False]) -> None: ...
    @property
    def annotationsFutureEnabled(self) -> bool: ...
    @annotationsFutureEnabled.setter
    def annotationsFutureEnabled(self, value: Literal[True]) -> None: ...
    @property
    def scope(self) -> Scope: ...
    @contextmanager
    def in_scope(self, cls: Callable[[], Scope]) -> Iterator[None]: ...
    def checkDeadScopes(self) -> None: ...
    def report(self, messageClass: Callable[_P, Message], *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    def getParent(self, node: ast.AST) -> ast.AST: ...
    def getCommonAncestor(self, lnode: ast.AST, rnode: ast.AST, stop: ast.AST) -> ast.AST: ...
    def descendantOf(self, node: ast.AST, ancestors: ast.AST, stop: ast.AST) -> bool: ...
    def getScopeNode(self, node: ast.AST) -> ast.AST | None: ...
    def differentForks(self, lnode: ast.AST, rnode: ast.AST) -> bool: ...
    def addBinding(self, node: ast.AST, value: Binding) -> None: ...
    def getNodeHandler(self, node_class: type[ast.AST]): ...
    def handleNodeLoad(self, node: ast.AST, parent: ast.AST) -> None: ...
    def handleNodeStore(self, node: ast.AST) -> None: ...
    def handleNodeDelete(self, node: ast.AST) -> None: ...
    def handleChildren(self, tree: ast.AST, omit: _OmitType = None) -> None: ...
    def isLiteralTupleUnpacking(self, node: ast.AST) -> bool | None: ...
    def isDocstring(self, node: ast.AST) -> bool: ...
    def getDocstring(self, node: ast.AST) -> tuple[str, int] | tuple[None, None]: ...
    def handleNode(self, node: ast.AST | None, parent) -> None: ...
    def handleDoctests(self, node: ast.AST) -> None: ...
    def handleStringAnnotation(self, s: str, node: ast.AST, ref_lineno: int, ref_col_offset: int, err: type[Message]) -> None: ...
    def handle_annotation_always_deferred(self, annotation: ast.AST, parent: ast.AST) -> None: ...
    def handleAnnotation(self, annotation: ast.AST, node: ast.AST) -> None: ...
    def ignore(self, node: ast.AST) -> None: ...
    def DELETE(self, tree: ast.Delete, omit: _OmitType = None) -> None: ...
    def FOR(self, tree: ast.For, omit: _OmitType = None) -> None: ...
    def ASYNCFOR(self, tree: ast.AsyncFor, omit: _OmitType = None) -> None: ...
    def WHILE(self, tree: ast.While, omit: _OmitType = None) -> None: ...
    def WITH(self, tree: ast.With, omit: _OmitType = None) -> None: ...
    def WITHITEM(self, tree: ast.AST, omit: _OmitType = None) -> None: ...
    def ASYNCWITH(self, tree: ast.AsyncWith, omit: _OmitType = None) -> None: ...
    def EXPR(self, tree: ast.AST, omit: _OmitType = None) -> None: ...
    def ASSIGN(self, tree: ast.Assign, omit: _OmitType = None) -> None: ...
    def PASS(self, node: ast.AST) -> None: ...
    def BOOLOP(self, tree: ast.BoolOp, omit: _OmitType = None) -> None: ...
    def UNARYOP(self, tree: ast.UnaryOp, omit: _OmitType = None) -> None: ...
    def SET(self, tree: ast.Set, omit: _OmitType = None) -> None: ...
    def ATTRIBUTE(self, tree: ast.Attribute, omit: _OmitType = None) -> None: ...
    def STARRED(self, tree: ast.Starred, omit: _OmitType = None) -> None: ...
    def NAMECONSTANT(self, tree: ast.NameConstant, omit: _OmitType = None) -> None: ...
    def NAMEDEXPR(self, tree: ast.NamedExpr, omit: _OmitType = None) -> None: ...
    def SUBSCRIPT(self, node: ast.Subscript) -> None: ...
    def CALL(self, node: ast.Call) -> None: ...
    def BINOP(self, node: ast.BinOp) -> None: ...
    def CONSTANT(self, node: ast.Constant) -> None: ...
    def SLICE(self, tree: ast.Slice, omit: _OmitType = None) -> None: ...
    def EXTSLICE(self, tree: ast.ExtSlice, omit: _OmitType = None) -> None: ...
    def INDEX(self, tree: ast.Index, omit: _OmitType = None) -> None: ...
    def LOAD(self, node: ast.Load) -> None: ...
    def STORE(self, node: ast.Store) -> None: ...
    def DEL(self, node: ast.Del) -> None: ...
    def AUGLOAD(self, node: ast.AugLoad) -> None: ...
    def AUGSTORE(self, node: ast.AugStore) -> None: ...
    def PARAM(self, node: ast.Param) -> None: ...
    def AND(self, node: ast.And) -> None: ...
    def OR(self, node: ast.Or) -> None: ...
    def ADD(self, node: ast.Add) -> None: ...
    def SUB(self, node: ast.Sub) -> None: ...
    def MULT(self, node: ast.Mult) -> None: ...
    def DIV(self, node: ast.Div) -> None: ...
    def MOD(self, node: ast.Mod) -> None: ...
    def POW(self, node: ast.Pow) -> None: ...
    def LSHIFT(self, node: ast.LShift) -> None: ...
    def RSHIFT(self, node: ast.RShift) -> None: ...
    def BITOR(self, node: ast.BitOr) -> None: ...
    def BITXOR(self, node: ast.BitXor) -> None: ...
    def BITAND(self, node: ast.BitAnd) -> None: ...
    def FLOORDIV(self, node: ast.FloorDiv) -> None: ...
    def INVERT(self, node: ast.Invert) -> None: ...
    def NOT(self, node: ast.Not) -> None: ...
    def UADD(self, node: ast.UAdd) -> None: ...
    def USUB(self, node: ast.USub) -> None: ...
    def EQ(self, node: ast.Eq) -> None: ...
    def NOTEQ(self, node: ast.NotEq) -> None: ...
    def LT(self, node: ast.Lt) -> None: ...
    def LTE(self, node: ast.LtE) -> None: ...
    def GT(self, node: ast.Gt) -> None: ...
    def GTE(self, node: ast.GtE) -> None: ...
    def IS(self, node: ast.Is) -> None: ...
    def ISNOT(self, node: ast.IsNot) -> None: ...
    def IN(self, node: ast.In) -> None: ...
    def NOTIN(self, node: ast.NotIn) -> None: ...
    def MATMULT(self, node: ast.MatMult) -> None: ...
    def RAISE(self, node: ast.Raise) -> None: ...
    def COMPREHENSION(self, tree: ast.comprehension, omit: _OmitType = None) -> None: ...
    def KEYWORD(self, tree: ast.keyword, omit: _OmitType = None) -> None: ...
    def FORMATTEDVALUE(self, tree: ast.FormattedValue, omit: _OmitType = None) -> None: ...
    def JOINEDSTR(self, node: ast.AST) -> None: ...
    def DICT(self, node: ast.Dict) -> None: ...
    def IF(self, node: ast.If) -> None: ...
    def IFEXP(self, node: ast.If) -> None: ...
    def ASSERT(self, node: ast.Assert) -> None: ...
    def GLOBAL(self, node: ast.Global) -> None: ...
    def NONLOCAL(self, node: ast.Nonlocal) -> None: ...
    def GENERATOREXP(self, node: ast.GeneratorExp) -> None: ...
    def LISTCOMP(self, node: ast.ListComp) -> None: ...
    def DICTCOMP(self, node: ast.DictComp) -> None: ...
    def SETCOMP(self, node: ast.SetComp) -> None: ...
    def NAME(self, node: ast.Name) -> None: ...
    def CONTINUE(self, node: ast.Continue) -> None: ...
    def BREAK(self, node: ast.Break) -> None: ...
    def RETURN(self, node: ast.Return) -> None: ...
    def YIELD(self, node: ast.Yield) -> None: ...
    def AWAIT(self, node: ast.Await) -> None: ...
    def YIELDFROM(self, node: ast.YieldFrom) -> None: ...
    def FUNCTIONDEF(self, node: ast.FunctionDef) -> None: ...
    def ASYNCFUNCTIONDEF(self, node: ast.AsyncFunctionDef) -> None: ...
    def LAMBDA(self, node: ast.Lambda) -> None: ...
    def ARGUMENTS(self, node: ast.arguments) -> None: ...
    def ARG(self, node: ast.arg) -> None: ...
    def CLASSDEF(self, node: ast.ClassDef): ...
    def AUGASSIGN(self, node: ast.AugAssign) -> None: ...
    def TUPLE(self, node: ast.Tuple) -> None: ...
    def LIST(self, node: ast.List) -> None: ...
    def IMPORT(self, node: ast.Import) -> None: ...
    def IMPORTFROM(self, node: ast.ImportFrom) -> None: ...
    def TRY(self, node: ast.Try) -> None: ...
    def EXCEPTHANDLER(self, node: ast.ExceptHandler) -> None: ...
    def ANNASSIGN(self, node: ast.AnnAssign) -> None: ...
    def COMPARE(self, node: ast.Compare) -> None: ...
    def MATCH(self, tree: _Match, omit: _OmitType = None) -> None: ...
    def MATCH_CASE(self, tree: _MatchCase, omit: _OmitType = None) -> None: ...
    def MATCHCLASS(self, tree: _MatchClass, omit: _OmitType = None) -> None: ...
    def MATCHOR(self, tree: _MatchOr, omit: _OmitType = None) -> None: ...
    def MATCHSEQUENCE(self, tree: _MatchSequence, omit: _OmitType = None) -> None: ...
    def MATCHSINGLETON(self, tree: _MatchSingleton, omit: _OmitType = None) -> None: ...
    def MATCHVALUE(self, tree: _MatchValue, omit: _OmitType = None) -> None: ...
    def MATCHAS(self, node: _MatchAs) -> None: ...
    def MATCHMAPPING(self, node: _MatchMapping) -> None: ...
    def MATCHSTAR(self, node: _MatchStar) -> None: ...
    def TYPEVAR(self, node: _TypeVar) -> None: ...
    def TYPEALIAS(self, node: _TypeAlias) -> None: ...



from typing import Any

from gevent.hub import Hub
from gevent.resolver import AbstractResolver

class Resolver(AbstractResolver):
    def __init__(self, hub: Hub | None = ...) -> None: ...
    @property
    def resolver(self) -> Any: ...  # this is a custom dnspython Resolver



from _typeshed import Incomplete
from typing import Any

from humanfriendly.compat import HTMLParser

def html_to_ansi(data, callback: Incomplete | None = None): ...

class HTMLConverter(HTMLParser):
    BLOCK_TAGS: Any
    callback: Any
    output: Any
    def __init__(self, *args, **kw) -> None: ...
    def __call__(self, data): ...
    @property
    def current_style(self): ...
    stack: Any
    def close(self) -> None: ...
    def emit_style(self, style: Incomplete | None = None) -> None: ...
    def handle_charref(self, value) -> None: ...
    link_text: Any
    def handle_data(self, data) -> None: ...
    def handle_endtag(self, tag) -> None: ...
    def handle_entityref(self, name) -> None: ...
    link_url: Any
    def handle_starttag(self, tag, attrs) -> None: ...
    def normalize_url(self, url): ...
    def parse_color(self, value): ...
    def push_styles(self, **changes) -> None: ...
    def render_url(self, url): ...
    preformatted_text_level: int
    def reset(self) -> None: ...
    def urls_match(self, a, b): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class UsersService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def delete_users_id(self, user_id, **kwargs): ...
    def delete_users_id_with_http_info(self, user_id, **kwargs): ...
    async def delete_users_id_async(self, user_id, **kwargs): ...
    def get_me(self, **kwargs): ...
    def get_me_with_http_info(self, **kwargs): ...
    async def get_me_async(self, **kwargs): ...
    def get_users(self, **kwargs): ...
    def get_users_with_http_info(self, **kwargs): ...
    async def get_users_async(self, **kwargs): ...
    def get_users_id(self, user_id, **kwargs): ...
    def get_users_id_with_http_info(self, user_id, **kwargs): ...
    async def get_users_id_async(self, user_id, **kwargs): ...
    def patch_users_id(self, user_id, user, **kwargs): ...
    def patch_users_id_with_http_info(self, user_id, user, **kwargs): ...
    async def patch_users_id_async(self, user_id, user, **kwargs): ...
    def post_users(self, user, **kwargs): ...
    def post_users_with_http_info(self, user, **kwargs): ...
    async def post_users_async(self, user, **kwargs): ...
    def post_users_id_password(self, user_id, password_reset_body, **kwargs): ...
    def post_users_id_password_with_http_info(self, user_id, password_reset_body, **kwargs): ...
    async def post_users_id_password_async(self, user_id, password_reset_body, **kwargs): ...
    def put_me_password(self, password_reset_body, **kwargs): ...
    def put_me_password_with_http_info(self, password_reset_body, **kwargs): ...
    async def put_me_password_async(self, password_reset_body, **kwargs): ...
    def put_users_id_password(self, user_id, password_reset_body, **kwargs): ...
    def put_users_id_password_with_http_info(self, user_id, password_reset_body, **kwargs): ...
    async def put_users_id_password_async(self, user_id, password_reset_body, **kwargs): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

def node_link_data(
    G, *, source: str = "source", target: str = "target", name: str = "id", key: str = "key", link: str = "links"
): ...
@_dispatch
def node_link_graph(
    data,
    directed: bool = False,
    multigraph: bool = True,
    attrs: Incomplete | None = None,
    *,
    source: str = "source",
    target: str = "target",
    name: str = "id",
    key: str = "key",
    link: str = "links",
): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def laplacian_spectrum(G, weight: str = "weight"): ...
@_dispatch
def normalized_laplacian_spectrum(G, weight: str = "weight"): ...
@_dispatch
def adjacency_spectrum(G, weight: str = "weight"): ...
@_dispatch
def modularity_spectrum(G): ...
@_dispatch
def bethe_hessian_spectrum(G, r: Incomplete | None = None): ...



from _typeshed import Incomplete
from collections.abc import Iterator
from dataclasses import dataclass

from networkx.classes.graph import _Node
from networkx.classes.multidigraph import MultiDiGraph
from networkx.utils.backends import _dispatch

__all__ = [
    "branching_weight",
    "greedy_branching",
    "maximum_branching",
    "minimum_branching",
    "maximum_spanning_arborescence",
    "minimum_spanning_arborescence",
    "ArborescenceIterator",
    "Edmonds",
]

@_dispatch
def branching_weight(G, attr: str = "weight", default: float = 1): ...
@_dispatch
def greedy_branching(G, attr: str = "weight", default: float = 1, kind: str = "max", seed: Incomplete | None = None): ...

class MultiDiGraph_EdgeKey(MultiDiGraph[_Node]):
    edge_index: Incomplete
    def __init__(self, incoming_graph_data: Incomplete | None = None, **attr) -> None: ...
    def remove_node(self, n) -> None: ...
    def remove_nodes_from(self, nbunch) -> None: ...
    def add_edge(self, u_for_edge, v_for_edge, key_for_edge, **attr) -> None: ...  # type: ignore[override]  # Graph.add_edge won't accept a `key_for_edge` keyword argument.
    def add_edges_from(self, ebunch_to_add, **attr) -> None: ...
    def remove_edge_with_key(self, key) -> None: ...
    def remove_edges_from(self, ebunch) -> None: ...

class Edmonds:
    G_original: Incomplete
    store: bool
    edges: Incomplete
    template: Incomplete
    def __init__(self, G, seed: Incomplete | None = None) -> None: ...
    def find_optimum(
        self,
        attr: str = "weight",
        default: float = 1,
        kind: str = "max",
        style: str = "branching",
        preserve_attrs: bool = False,
        partition: Incomplete | None = None,
        seed: Incomplete | None = None,
    ): ...

@_dispatch
def maximum_branching(
    G, attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: Incomplete | None = None
): ...
@_dispatch
def minimum_branching(
    G, attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: Incomplete | None = None
): ...
@_dispatch
def maximum_spanning_arborescence(
    G, attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: Incomplete | None = None
): ...
@_dispatch
def minimum_spanning_arborescence(
    G, attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: Incomplete | None = None
): ...

class ArborescenceIterator:
    @dataclass
    class Partition:
        mst_weight: float
        partition_dict: dict[Incomplete, Incomplete]

    G: Incomplete
    weight: Incomplete
    minimum: Incomplete
    method: Incomplete
    partition_key: str
    init_partition: Incomplete
    def __init__(self, G, weight: str = "weight", minimum: bool = True, init_partition: Incomplete | None = None) -> None: ...
    partition_queue: Incomplete
    def __iter__(self) -> Iterator[Incomplete]: ...
    def __next__(self): ...



from networkx.utils.backends import _dispatch

@_dispatch
def trophic_levels(G, weight: str = "weight"): ...
@_dispatch
def trophic_differences(G, weight: str = "weight"): ...
@_dispatch
def trophic_incoherence_parameter(G, weight: str = "weight", cannibalism: bool = False): ...



class JOSEError(Exception): ...
class JWSError(JOSEError): ...
class JWSSignatureError(JWSError): ...
class JWSAlgorithmError(JWSError): ...
class JWTError(JOSEError): ...
class JWTClaimsError(JWTError): ...
class ExpiredSignatureError(JWTError): ...
class JWKError(JOSEError): ...
class JWEError(JOSEError): ...
class JWEParseError(JWEError): ...
class JWEInvalidAuth(JWEError): ...
class JWEAlgorithmUnsupportedError(JWEError): ...



from typing import overload

from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
)

class PaymentLink(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str

    @overload
    @classmethod
    def list_line_items(
        cls,
        payment_link: str,
        api_key: str | None = None,
        stripe_version: str | None = None,
        stripe_account: str | None = None,
        **params,
    ): ...
    @overload
    @classmethod
    def list_line_items(cls, idempotency_key: str | None = None, **params): ...



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Radius(VaultApiBase):
    def configure(
        self,
        host,
        secret,
        port: Incomplete | None = None,
        unregistered_user_policies: Incomplete | None = None,
        dial_timeout: Incomplete | None = None,
        nas_port: Incomplete | None = None,
        mount_point="radius",
    ): ...
    def read_configuration(self, mount_point="radius"): ...
    def register_user(self, username, policies: Incomplete | None = None, mount_point="radius"): ...
    def list_users(self, mount_point="radius"): ...
    def read_user(self, username, mount_point="radius"): ...
    def delete_user(self, username, mount_point="radius"): ...
    def login(self, username, password, use_token: bool = True, mount_point="radius"): ...



from _typeshed import Incomplete

def powerlaw_sequence(n, exponent: float = 2.0, seed: Incomplete | None = None): ...
def zipf_rv(alpha, xmin: int = 1, seed: Incomplete | None = None): ...
def cumulative_distribution(distribution): ...
def discrete_sequence(
    n, distribution: Incomplete | None = None, cdistribution: Incomplete | None = None, seed: Incomplete | None = None
): ...
def random_weighted_sample(mapping, k, seed: Incomplete | None = None): ...
def weighted_choice(mapping, seed: Incomplete | None = None): ...



from typing import Any, ClassVar, Literal
from typing_extensions import TypeAlias

from .TiffImagePlugin import TiffImageFile

_OleFileIO: TypeAlias = Any  # olefile.OleFileIO
_OleStream: TypeAlias = Any  # olefile.OleStream

class MicImageFile(TiffImageFile):
    ole: _OleFileIO
    format: ClassVar[Literal["MIC"]]
    format_description: ClassVar[str]
    fp: _OleStream
    frame: int | None
    images: list[list[str]]
    is_animated: bool
    def seek(self, frame: int) -> None: ...
    def tell(self) -> int | None: ...  # type: ignore[override]



from collections.abc import Callable, Container, Iterable, Sequence
from re import Pattern
from typing import Literal

from docutils.languages import _LanguageModule
from docutils.nodes import document
from docutils.parsers import Parser
from docutils.parsers.rst import Directive
from docutils.utils import SystemMessage

def register_directive(name: str, directive: type[Directive]) -> None: ...
def directive(
    directive_name: str, language_module: _LanguageModule, document: document
) -> tuple[type[Directive] | None, list[SystemMessage]]: ...
def flag(argument: str | None) -> None: ...
def unchanged_required(argument: str) -> str: ...
def unchanged(argument: str | None) -> str: ...
def path(argument: str) -> str: ...
def uri(argument: str) -> str: ...
def nonnegative_int(argument: str) -> int: ...
def percentage(argument: str) -> int: ...

length_units: list[str]

def get_measure(argument: str, units: Iterable[str]) -> str: ...
def length_or_unitless(argument: str) -> str: ...
def length_or_percentage_or_unitless(argument: str, default: str = "") -> str: ...
def class_option(argument: str) -> list[str]: ...

unicode_pattern: Pattern[str]

def unicode_code(code: str) -> str: ...
def single_char_or_unicode(argument: str) -> str: ...
def single_char_or_whitespace_or_unicode(argument: str | Literal["tab", "space"]) -> str: ...  # noqa: Y051
def positive_int(argument: str) -> int: ...
def positive_int_list(argument: str) -> list[int]: ...
def encoding(argument: str) -> str: ...
def choice(argument: str, values: Sequence[str]) -> str: ...
def format_values(values: Sequence[object]) -> str: ...
def value_or(values: Container[str], other: Callable[[str], str]) -> Callable[[str], str]: ...
def parser_name(argument: str | None) -> type[Parser] | None: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from typing import ClassVar

from ..lexer import RegexLexer
from ..token import _TokenType

__all__ = [
    "SchemeLexer",
    "CommonLispLexer",
    "HyLexer",
    "RacketLexer",
    "NewLispLexer",
    "EmacsLispLexer",
    "ShenLexer",
    "CPSALexer",
    "XtlangLexer",
    "FennelLexer",
]

class SchemeLexer(RegexLexer):
    valid_name: ClassVar[str]
    token_end: ClassVar[str]
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...  # type: ignore[override]
    number_rules: ClassVar[dict[Incomplete, Incomplete]]
    def decimal_cb(self, match) -> Iterator[tuple[Incomplete, Incomplete, Incomplete]]: ...

class CommonLispLexer(RegexLexer):
    nonmacro: ClassVar[str]
    constituent: ClassVar[str]
    terminated: ClassVar[str]
    symbol: ClassVar[str]
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...  # type: ignore[override]

class HyLexer(RegexLexer):
    special_forms: ClassVar[tuple[str, ...]]
    declarations: ClassVar[tuple[str, ...]]
    hy_builtins: ClassVar[tuple[str, ...]]
    hy_core: ClassVar[tuple[str, ...]]
    builtins: ClassVar[tuple[str, ...]]
    valid_name: ClassVar[str]

class RacketLexer(RegexLexer): ...

class NewLispLexer(RegexLexer):
    builtins: ClassVar[tuple[str, ...]]
    valid_name: ClassVar[str]

class EmacsLispLexer(RegexLexer):
    nonmacro: ClassVar[str]
    constituent: ClassVar[str]
    terminated: ClassVar[str]
    symbol: ClassVar[str]
    macros: ClassVar[set[str]]
    special_forms: ClassVar[set[str]]
    builtin_function: ClassVar[set[str]]
    builtin_function_highlighted: ClassVar[set[str]]
    lambda_list_keywords: ClassVar[set[str]]
    error_keywords: ClassVar[set[str]]
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...  # type: ignore[override]

class ShenLexer(RegexLexer):
    DECLARATIONS: ClassVar[tuple[str, ...]]
    SPECIAL_FORMS: ClassVar[tuple[str, ...]]
    BUILTINS: ClassVar[tuple[str, ...]]
    BUILTINS_ANYWHERE: ClassVar[tuple[str, ...]]
    MAPPINGS: ClassVar[dict[str, Incomplete]]

    valid_symbol_chars: ClassVar[str]
    valid_name: ClassVar[str]
    symbol_name: ClassVar[str]
    variable: ClassVar[str]

    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...  # type: ignore[override]

class CPSALexer(RegexLexer):
    valid_name: ClassVar[str]

class XtlangLexer(RegexLexer):
    common_keywords: ClassVar[tuple[str, ...]]
    scheme_keywords: ClassVar[tuple[str, ...]]
    xtlang_bind_keywords: ClassVar[tuple[str, ...]]
    xtlang_keywords: ClassVar[tuple[str, ...]]
    common_functions: ClassVar[tuple[str, ...]]
    scheme_functions: ClassVar[tuple[str, ...]]
    xtlang_functions: ClassVar[tuple[str, ...]]

    valid_scheme_name: ClassVar[str]
    valid_xtlang_name: ClassVar[str]
    valid_xtlang_type: ClassVar[str]

class FennelLexer(RegexLexer):
    special_forms: ClassVar[tuple[str, ...]]
    declarations: ClassVar[tuple[str, ...]]
    builtins: ClassVar[tuple[str, ...]]
    valid_name: ClassVar[str]



from typing import ClassVar

from .core import UnitedStates

class Tennessee(UnitedStates):
    include_columbus_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]



from braintree.search import Search

class SubscriptionSearch:
    billing_cycles_remaining: Search.RangeNodeBuilder
    created_at: Search.RangeNodeBuilder
    days_past_due: Search.RangeNodeBuilder
    id: Search.TextNodeBuilder
    ids: Search.MultipleValueNodeBuilder
    in_trial_period: Search.MultipleValueNodeBuilder
    merchant_account_id: Search.MultipleValueNodeBuilder
    next_billing_date: Search.RangeNodeBuilder
    plan_id: Search.MultipleValueOrTextNodeBuilder
    price: Search.RangeNodeBuilder
    status: Search.MultipleValueNodeBuilder
    transaction_id: Search.TextNodeBuilder



from _typeshed import Incomplete, Unused
from collections import defaultdict
from logging import Logger
from typing import Final

from .annotations import AnnotationDict
from .encryption import StandardSecurityHandler
from .fpdf import FPDF
from .image_datastructures import RasterImageInfo
from .syntax import Name, PDFArray, PDFContentStream, PDFObject, PDFString

LOGGER: Logger
ZOOM_CONFIGS: Final[dict[str, tuple[str, ...]]]

class ContentWithoutID:
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str | None: ...

class PDFHeader(ContentWithoutID):
    pdf_version: str
    def __init__(self, pdf_version: str) -> None: ...
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class PDFFont(PDFObject):
    type: Name
    subtype: Name
    base_font: Name
    encoding: Name | None
    d_w: Incomplete | None
    w: Incomplete | None
    descendant_fonts: Incomplete | None
    to_unicode: Incomplete | None
    c_i_d_system_info: Incomplete | None
    font_descriptor: Incomplete | None
    c_i_d_to_g_i_d_map: Incomplete | None
    def __init__(
        self,
        subtype: str,
        base_font: str,
        encoding: str | None = None,
        d_w: Incomplete | None = None,
        w: Incomplete | None = None,
    ) -> None: ...

class CIDSystemInfo(PDFObject):
    registry: PDFString
    ordering: PDFString
    supplement: int

class PDFInfo(PDFObject):
    title: str | None
    subject: str | None
    author: str | None
    keywords: str | None
    creator: str | None
    producer: str | None
    creation_date: Incomplete
    def __init__(
        self,
        title: str | None,
        subject: str | None,
        author: str | None,
        keywords: str | None,
        creator: str | None,
        producer: str | None,
        creation_date,
    ) -> None: ...

class AcroForm:
    fields: Incomplete
    sig_flags: Incomplete
    def __init__(self, fields, sig_flags) -> None: ...
    def serialize(self) -> str: ...

class PDFCatalog(PDFObject):
    type: Name
    lang: str | None
    page_layout: Incomplete | None
    page_mode: Incomplete | None
    viewer_preferences: Incomplete | None
    pages: Incomplete | None
    acro_form: Incomplete | None
    open_action: Incomplete | None
    mark_info: Incomplete | None
    metadata: Incomplete | None
    names: Incomplete | None
    outlines: Incomplete | None
    struct_tree_root: Incomplete | None
    def __init__(
        self,
        lang: str | None = None,
        page_layout: Incomplete | None = None,
        page_mode: Incomplete | None = None,
        viewer_preferences: Incomplete | None = None,
    ) -> None: ...

class PDFResources(PDFObject):
    proc_set: Incomplete
    font: Incomplete
    x_object: Incomplete
    ext_g_state: Incomplete
    def __init__(self, proc_set, font, x_object, ext_g_state) -> None: ...

class PDFFontStream(PDFContentStream):
    length1: int
    def __init__(self, contents: bytes) -> None: ...

class PDFXmpMetadata(PDFContentStream):
    type: Name
    subtype: Name
    def __init__(self, contents: bytes) -> None: ...

class PDFXObject(PDFContentStream):
    type: Name
    subtype: Name
    width: Incomplete
    height: Incomplete
    color_space: Incomplete
    bits_per_component: Incomplete
    filter: Name
    decode: Incomplete | None
    decode_parms: Incomplete | None
    s_mask: Incomplete | None
    def __init__(
        self,
        contents,
        subtype: str,
        width,
        height,
        color_space,
        bits_per_component,
        img_filter: str | None = None,
        decode: Incomplete | None = None,
        decode_parms: Incomplete | None = None,
    ) -> None: ...

class PDFICCPObject(PDFContentStream):
    n: Incomplete
    alternate: Name
    def __init__(self, contents: bytes, n, alternate: str) -> None: ...

class PDFPage(PDFObject):
    type: Name
    contents: Incomplete
    dur: Incomplete | None
    trans: Incomplete
    annots: PDFArray[AnnotationDict]
    group: Incomplete | None
    media_box: Incomplete | None
    struct_parents: Incomplete | None
    resources: Incomplete | None
    parent: Incomplete | None
    def __init__(self, duration: Incomplete | None, transition, contents, index) -> None: ...
    def index(self): ...
    def dimensions(self) -> tuple[float | None, float | None]: ...
    def set_dimensions(self, width_pt: float | None, height_pt: float | None) -> None: ...

class PDFPagesRoot(PDFObject):
    type: Name
    count: Incomplete
    media_box: Incomplete
    kids: Incomplete | None
    def __init__(self, count, media_box) -> None: ...

class PDFExtGState(PDFObject):
    def __init__(self, dict_as_str) -> None: ...
    def serialize(self, obj_dict: Unused = None, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class PDFXrefAndTrailer(ContentWithoutID):
    output_builder: Incomplete
    count: int
    catalog_obj: Incomplete | None
    info_obj: Incomplete | None
    def __init__(self, output_builder) -> None: ...
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class OutputProducer:
    fpdf: FPDF
    pdf_objs: list[Incomplete]
    obj_id: int
    offsets: dict[Incomplete, Incomplete]
    trace_labels_per_obj_id: dict[Incomplete, Incomplete]
    sections_size_per_trace_label: defaultdict[Incomplete, int]
    buffer: bytearray
    def __init__(self, fpdf: FPDF) -> None: ...
    def bufferize(self) -> bytearray: ...

def stream_content_for_raster_image(
    info: RasterImageInfo,
    x: float,
    y: float,
    w: float,
    h: float,
    keep_aspect_ratio: bool = False,
    scale: float = 1,
    pdf_height_to_flip: float | None = None,
) -> str: ...



from pyasn1.type import univ
from pyasn1.type.tag import TagSet

__all__ = [
    "NumericString",
    "PrintableString",
    "TeletexString",
    "T61String",
    "VideotexString",
    "IA5String",
    "GraphicString",
    "VisibleString",
    "ISO646String",
    "GeneralString",
    "UniversalString",
    "BMPString",
    "UTF8String",
]

class AbstractCharacterString(univ.OctetString):
    def __bytes__(self) -> bytes: ...
    def prettyIn(self, value): ...
    def asOctets(self, padding: bool = True): ...
    def asNumbers(self, padding: bool = True): ...
    def prettyOut(self, value): ...
    def prettyPrint(self, scope: int = 0): ...
    def __reversed__(self): ...

class NumericString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class PrintableString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class TeletexString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class T61String(TeletexString):
    typeId: int

class VideotexString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class IA5String(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class GraphicString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class VisibleString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class ISO646String(VisibleString):
    typeId: int

class GeneralString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class UniversalString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class BMPString(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int

class UTF8String(AbstractCharacterString):
    tagSet: TagSet
    encoding: str
    typeId: int



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Raft(SystemBackendMixin):
    def join_raft_cluster(
        self,
        leader_api_addr,
        retry: bool = False,
        leader_ca_cert: Incomplete | None = None,
        leader_client_cert: Incomplete | None = None,
        leader_client_key: Incomplete | None = None,
    ): ...
    def read_raft_config(self): ...
    def remove_raft_node(self, server_id): ...
    def take_raft_snapshot(self): ...
    def restore_raft_snapshot(self, snapshot): ...
    def force_restore_raft_snapshot(self, snapshot): ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable, Iterable, Sequence
from threading import Lock
from types import TracebackType
from typing import Any, ClassVar, Literal, NoReturn, Protocol
from typing_extensions import Self

from redis.client import CaseInsensitiveDict, PubSub, Redis, _ParseResponseOptions
from redis.commands import CommandsParser, RedisClusterCommands
from redis.commands.core import _StrType
from redis.connection import BaseParser, Connection, ConnectionPool, Encoder, _ConnectionPoolOptions, _Encodable
from redis.exceptions import MovedError, RedisError
from redis.retry import Retry
from redis.typing import EncodableT

def get_node_name(host: str, port: str | int) -> str: ...
def get_connection(redis_node: Redis[Any], *args, **options: _ConnectionPoolOptions) -> Connection: ...
def parse_scan_result(command: Unused, res, **options): ...
def parse_pubsub_numsub(command: Unused, res, **options: Unused): ...
def parse_cluster_slots(resp, **options) -> dict[tuple[int, int], dict[str, Any]]: ...
def parse_cluster_myshardid(resp: bytes, **options: Unused) -> str: ...

PRIMARY: str
REPLICA: str
SLOT_ID: str
REDIS_ALLOWED_KEYS: tuple[str, ...]
KWARGS_DISABLED_KEYS: tuple[str, ...]
PIPELINE_BLOCKED_COMMANDS: tuple[str, ...]

def cleanup_kwargs(**kwargs: Any) -> dict[str, Any]: ...

# It uses `DefaultParser` in real life, but it is a dynamic base class.
class ClusterParser(BaseParser): ...

class AbstractRedisCluster:
    RedisClusterRequestTTL: ClassVar[int]
    PRIMARIES: ClassVar[str]
    REPLICAS: ClassVar[str]
    ALL_NODES: ClassVar[str]
    RANDOM: ClassVar[str]
    DEFAULT_NODE: ClassVar[str]
    NODE_FLAGS: ClassVar[set[str]]
    COMMAND_FLAGS: ClassVar[dict[str, str]]
    CLUSTER_COMMANDS_RESPONSE_CALLBACKS: ClassVar[dict[str, Any]]
    RESULT_CALLBACKS: ClassVar[dict[str, Callable[[Incomplete, Incomplete], Incomplete]]]
    ERRORS_ALLOW_RETRY: ClassVar[tuple[type[RedisError], ...]]

class RedisCluster(AbstractRedisCluster, RedisClusterCommands[_StrType]):
    user_on_connect_func: Callable[[Connection], object] | None
    encoder: Encoder
    cluster_error_retry_attempts: int
    command_flags: dict[str, str]
    node_flags: set[str]
    read_from_replicas: bool
    reinitialize_counter: int
    reinitialize_steps: int
    nodes_manager: NodesManager
    cluster_response_callbacks: CaseInsensitiveDict[str, Callable[..., Incomplete]]
    result_callbacks: CaseInsensitiveDict[str, Callable[[Incomplete, Incomplete], Incomplete]]
    commands_parser: CommandsParser
    def __init__(  # TODO: make @overloads, either `url` or `host:port` can be passed
        self,
        host: str | None = None,
        port: int | None = 6379,
        startup_nodes: list[ClusterNode] | None = None,
        cluster_error_retry_attempts: int = 3,
        retry: Retry | None = None,
        require_full_coverage: bool = False,
        reinitialize_steps: int = 5,
        read_from_replicas: bool = False,
        dynamic_startup_nodes: bool = True,
        url: str | None = None,
        address_remap: Callable[[str, int], tuple[str, int]] | None = None,
        **kwargs,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...
    def disconnect_connection_pools(self) -> None: ...
    @classmethod
    def from_url(cls, url: str, **kwargs) -> Self: ...
    def on_connect(self, connection: Connection) -> None: ...
    def get_redis_connection(self, node: ClusterNode) -> Redis[Any]: ...
    def get_node(
        self, host: str | None = None, port: str | int | None = None, node_name: str | None = None
    ) -> ClusterNode | None: ...
    def get_primaries(self) -> list[ClusterNode]: ...
    def get_replicas(self) -> list[ClusterNode]: ...
    def get_random_node(self) -> ClusterNode: ...
    def get_nodes(self) -> list[ClusterNode]: ...
    def get_node_from_key(self, key: _Encodable, replica: bool = False) -> ClusterNode | None: ...
    def get_default_node(self) -> ClusterNode | None: ...
    def set_default_node(self, node: ClusterNode | None) -> bool: ...
    def monitor(self, target_node: Incomplete | None = None): ...
    def pubsub(
        self, node: Incomplete | None = None, host: Incomplete | None = None, port: Incomplete | None = None, **kwargs
    ): ...
    def pipeline(self, transaction: Incomplete | None = None, shard_hint: Incomplete | None = None): ...
    def lock(
        self,
        name: str,
        timeout: float | None = None,
        sleep: float = 0.1,
        blocking: bool = True,
        blocking_timeout: float | None = None,
        lock_class: type[Incomplete] | None = None,
        thread_local: bool = True,
    ): ...
    def keyslot(self, key: _Encodable) -> int: ...
    def determine_slot(self, *args): ...
    def get_encoder(self) -> Encoder: ...
    def get_connection_kwargs(self) -> dict[str, Any]: ...
    def execute_command(self, *args, **kwargs): ...
    def close(self) -> None: ...

class ClusterNode:
    host: str
    port: int
    name: str
    server_type: str | None
    redis_connection: Redis[Incomplete] | None
    def __init__(
        self, host: str, port: int, server_type: str | None = None, redis_connection: Redis[Incomplete] | None = None
    ) -> None: ...
    def __eq__(self, obj: object) -> bool: ...
    def __del__(self) -> None: ...

class LoadBalancer:
    primary_to_idx: dict[str, int]
    start_index: int
    def __init__(self, start_index: int = 0) -> None: ...
    def get_server_index(self, primary: str, list_size: int) -> int: ...
    def reset(self) -> None: ...

class NodesManager:
    nodes_cache: dict[str, ClusterNode]
    slots_cache: dict[str, list[ClusterNode]]
    startup_nodes: dict[str, ClusterNode]
    default_node: ClusterNode | None
    from_url: bool
    connection_pool_class: type[ConnectionPool]
    connection_kwargs: dict[str, Incomplete]  # TODO: could be a TypedDict
    read_load_balancer: LoadBalancer
    address_remap: Callable[[str, int], tuple[str, int]] | None
    def __init__(
        self,
        startup_nodes: Iterable[ClusterNode],
        from_url: bool = False,
        require_full_coverage: bool = False,
        lock: Lock | None = None,
        dynamic_startup_nodes: bool = True,
        connection_pool_class: type[ConnectionPool] = ...,
        address_remap: Callable[[str, int], tuple[str, int]] | None = None,
        **kwargs,  # TODO: same type as connection_kwargs
    ) -> None: ...
    def get_node(
        self, host: str | None = None, port: int | str | None = None, node_name: str | None = None
    ) -> ClusterNode | None: ...
    def update_moved_exception(self, exception: MovedError) -> None: ...
    def get_node_from_slot(self, slot: str, read_from_replicas: bool = False, server_type: str | None = None) -> ClusterNode: ...
    def get_nodes_by_server_type(self, server_type: str) -> list[ClusterNode]: ...
    def populate_startup_nodes(self, nodes: Iterable[ClusterNode]) -> None: ...
    def check_slots_coverage(self, slots_cache: dict[str, list[ClusterNode]]) -> bool: ...
    def create_redis_connections(self, nodes: Iterable[ClusterNode]) -> None: ...
    def create_redis_node(self, host: str, port: int | str, **kwargs: Any) -> Redis[Incomplete]: ...
    def initialize(self) -> None: ...
    def close(self) -> None: ...
    def reset(self) -> None: ...
    def remap_host_port(self, host: str, port: int) -> tuple[str, int]: ...

class ClusterPubSub(PubSub):
    node: ClusterNode | None
    cluster: RedisCluster[Any]
    def __init__(
        self,
        redis_cluster: RedisCluster[Any],
        node: ClusterNode | None = None,
        host: str | None = None,
        port: int | None = None,
        **kwargs,
    ) -> None: ...
    def set_pubsub_node(
        self, cluster: RedisCluster[Any], node: ClusterNode | None = None, host: str | None = None, port: int | None = None
    ) -> None: ...
    def get_pubsub_node(self) -> ClusterNode | None: ...
    def execute_command(self, *args, **kwargs) -> None: ...
    def get_redis_connection(self) -> Redis[Any] | None: ...

class ClusterPipeline(RedisCluster[_StrType]):
    command_stack: list[Incomplete]
    nodes_manager: Incomplete
    refresh_table_asap: bool
    result_callbacks: Incomplete
    startup_nodes: Incomplete
    read_from_replicas: bool
    command_flags: Incomplete
    cluster_response_callbacks: Incomplete
    cluster_error_retry_attempts: int
    reinitialize_counter: int
    reinitialize_steps: int
    encoder: Encoder
    commands_parser: Incomplete
    def __init__(
        self,
        nodes_manager,
        commands_parser,
        result_callbacks: Incomplete | None = None,
        cluster_response_callbacks: Incomplete | None = None,
        startup_nodes: Incomplete | None = None,
        read_from_replicas: bool = False,
        cluster_error_retry_attempts: int = 3,
        reinitialize_steps: int = 5,
        lock: Lock | None = None,
        **kwargs,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> Literal[True]: ...
    def execute_command(self, *args, **kwargs): ...
    def pipeline_execute_command(self, *args, **options): ...
    def raise_first_error(self, stack) -> None: ...
    def annotate_exception(self, exception, number, command) -> None: ...
    def execute(self, raise_on_error: bool = True): ...
    scripts: set[Any]  # is only set in `reset()`
    watching: bool  # is only set in `reset()`
    explicit_transaction: bool  # is only set in `reset()`
    def reset(self) -> None: ...
    def send_cluster_commands(self, stack, raise_on_error: bool = True, allow_redirections: bool = True): ...
    def eval(self) -> None: ...
    def multi(self) -> None: ...
    def immediate_execute_command(self, *args, **options) -> None: ...
    def load_scripts(self) -> None: ...
    def watch(self, *names) -> None: ...
    def unwatch(self) -> None: ...
    def script_load_for_pipeline(self, *args, **kwargs) -> None: ...
    def delete(self, *names): ...

def block_pipeline_command(name: str) -> Callable[..., NoReturn]: ...

class PipelineCommand:
    args: Sequence[EncodableT]
    options: _ParseResponseOptions
    position: int | None
    result: Any | Exception | None
    node: Incomplete | None
    asking: bool
    def __init__(
        self, args: Sequence[EncodableT], options: _ParseResponseOptions | None = None, position: int | None = None
    ) -> None: ...

class _ParseResponseCallback(Protocol):
    def __call__(self, connection: Connection, command: EncodableT, /, **kwargs: Incomplete) -> Any: ...

class NodeCommands:
    parse_response: _ParseResponseCallback
    connection_pool: ConnectionPool
    connection: Connection
    commands: list[PipelineCommand]
    def __init__(
        self, parse_response: _ParseResponseCallback, connection_pool: ConnectionPool, connection: Connection
    ) -> None: ...
    def append(self, c: PipelineCommand) -> None: ...
    def write(self) -> None: ...
    def read(self) -> None: ...



from networkx.utils.backends import _dispatch

def write_p2g(G, path, encoding: str = "utf-8") -> None: ...
@_dispatch
def read_p2g(path, encoding: str = "utf-8"): ...
@_dispatch
def parse_p2g(lines): ...



from _typeshed import Incomplete

class ScraperTargetRequest:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        name: Incomplete | None = None,
        type: Incomplete | None = None,
        url: Incomplete | None = None,
        org_id: Incomplete | None = None,
        bucket_id: Incomplete | None = None,
        allow_insecure: bool = False,
    ) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, name) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type) -> None: ...
    @property
    def url(self): ...
    @url.setter
    def url(self, url) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def bucket_id(self): ...
    @bucket_id.setter
    def bucket_id(self, bucket_id) -> None: ...
    @property
    def allow_insecure(self): ...
    @allow_insecure.setter
    def allow_insecure(self, allow_insecure) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete
from types import TracebackType
from typing_extensions import Self

from influxdb_client import HealthCheck, InvokableScriptsApi, Ready
from influxdb_client.client._base import _BaseClient
from influxdb_client.client.authorizations_api import AuthorizationsApi
from influxdb_client.client.bucket_api import BucketsApi
from influxdb_client.client.delete_api import DeleteApi
from influxdb_client.client.labels_api import LabelsApi
from influxdb_client.client.organizations_api import OrganizationsApi
from influxdb_client.client.query_api import QueryApi, QueryOptions
from influxdb_client.client.tasks_api import TasksApi
from influxdb_client.client.users_api import UsersApi
from influxdb_client.client.write_api import PointSettings, WriteApi, WriteOptions

logger: Incomplete

class InfluxDBClient(_BaseClient):
    api_client: Incomplete
    def __init__(
        self,
        url: str,
        token: str | None = None,
        debug: bool | None = None,
        timeout: int = 10000,
        enable_gzip: bool = False,
        org: str | None = None,
        default_tags: dict[Incomplete, Incomplete] | None = None,
        *,
        verify_ssl: bool = ...,
        ssl_ca_cert: Incomplete | None = ...,
        cert_file: Incomplete | None = ...,
        cert_key_file: Incomplete | None = ...,
        cert_key_password: Incomplete | None = ...,
        ssl_context: Incomplete | None = ...,
        proxy: Incomplete | None = ...,
        proxy_headers: Incomplete | None = ...,
        connection_pool_maxsize: int = ...,
        username: Incomplete | None = ...,
        password: Incomplete | None = ...,
        auth_basic: bool = ...,
        retries: bool | Incomplete = ...,
        profilers: Incomplete | None = ...,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    @classmethod
    def from_config_file(
        cls, config_file: str = "config.ini", debug: Incomplete | None = None, enable_gzip: bool = False, **kwargs
    ): ...
    @classmethod
    def from_env_properties(cls, debug: Incomplete | None = None, enable_gzip: bool = False, **kwargs): ...
    def write_api(self, write_options: WriteOptions = ..., point_settings: PointSettings = ..., **kwargs) -> WriteApi: ...
    def query_api(self, query_options: QueryOptions = ...) -> QueryApi: ...
    def invokable_scripts_api(self) -> InvokableScriptsApi: ...
    def close(self) -> None: ...
    def __del__(self) -> None: ...
    def buckets_api(self) -> BucketsApi: ...
    def authorizations_api(self) -> AuthorizationsApi: ...
    def users_api(self) -> UsersApi: ...
    def organizations_api(self) -> OrganizationsApi: ...
    def tasks_api(self) -> TasksApi: ...
    def labels_api(self) -> LabelsApi: ...
    def health(self) -> HealthCheck: ...
    def ping(self) -> bool: ...
    def version(self) -> str: ...
    def build(self) -> str: ...
    def ready(self) -> Ready: ...
    def delete_api(self) -> DeleteApi: ...



from collections.abc import Hashable
from typing import TypeVar

from networkx.classes.graph import Graph
from networkx.utils.backends import _dispatch

_G = TypeVar("_G", bound=Graph[Hashable])

@_dispatch
def complement(G): ...
@_dispatch
def reverse(G: _G, copy: bool = True) -> _G: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def triangles(G, nodes: Incomplete | None = None): ...
@_dispatch
def average_clustering(G, nodes: Incomplete | None = None, weight: Incomplete | None = None, count_zeros: bool = True): ...
@_dispatch
def clustering(G, nodes: Incomplete | None = None, weight: Incomplete | None = None): ...
@_dispatch
def transitivity(G): ...
@_dispatch
def square_clustering(G, nodes: Incomplete | None = None): ...
@_dispatch
def generalized_degree(G, nodes: Incomplete | None = None): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

class _PCGSolver:
    def __init__(self, A, M) -> None: ...
    def solve(self, B, tol): ...

class _LUSolver:
    def __init__(self, A) -> None: ...
    def solve(self, B, tol: Incomplete | None = None): ...

@_dispatch
def algebraic_connectivity(
    G,
    weight: str = "weight",
    normalized: bool = False,
    tol: float = 1e-08,
    method: str = "tracemin_pcg",
    seed: Incomplete | None = None,
): ...
@_dispatch
def fiedler_vector(
    G,
    weight: str = "weight",
    normalized: bool = False,
    tol: float = 1e-08,
    method: str = "tracemin_pcg",
    seed: Incomplete | None = None,
): ...
@_dispatch
def spectral_ordering(
    G,
    weight: str = "weight",
    normalized: bool = False,
    tol: float = 1e-08,
    method: str = "tracemin_pcg",
    seed: Incomplete | None = None,
): ...
@_dispatch
def spectral_bisection(
    G,
    weight: str = "weight",
    normalized: bool = False,
    tol: float = 1e-08,
    method: str = "tracemin_pcg",
    seed: Incomplete | None = None,
): ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Sequence
from typing import Literal
from typing_extensions import Self, TypeAlias

from .Image import Image

_FilterArgs: TypeAlias = tuple[Sequence[int], int, int, Sequence[int]]

# filter image parameters below are the C images, i.e. Image().im.

class Filter: ...
class MultibandFilter(Filter): ...

class BuiltinFilter(MultibandFilter):
    def filter(self, image) -> Image: ...

class Kernel(BuiltinFilter):
    name: str
    filterargs: _FilterArgs
    def __init__(self, size: Sequence[int], kernel: Sequence[int], scale: Incomplete | None = None, offset: int = 0) -> None: ...

class RankFilter(Filter):
    name: str
    size: int
    rank: int
    def __init__(self, size: int, rank: int) -> None: ...
    def filter(self, image) -> Image: ...

class MedianFilter(RankFilter):
    name: str
    size: int
    rank: int
    def __init__(self, size: int = 3) -> None: ...

class MinFilter(RankFilter):
    name: str
    size: int
    rank: int
    def __init__(self, size: int = 3) -> None: ...

class MaxFilter(RankFilter):
    name: str
    size: int
    rank: int
    def __init__(self, size: int = 3) -> None: ...

class ModeFilter(Filter):
    name: str
    size: int
    def __init__(self, size: int = 3) -> None: ...
    def filter(self, image) -> Image: ...

class GaussianBlur(MultibandFilter):
    name: str
    radius: float
    def __init__(self, radius: float = 2) -> None: ...
    def filter(self, image) -> Image: ...

class BoxBlur(MultibandFilter):
    name: str
    radius: float
    def __init__(self, radius: float) -> None: ...
    def filter(self, image) -> Image: ...

class UnsharpMask(MultibandFilter):
    name: str
    radius: float
    percent: int
    threshold: int
    def __init__(self, radius: float = 2, percent: int = 150, threshold: int = 3) -> None: ...
    def filter(self, image) -> Image: ...

class BLUR(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class CONTOUR(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class DETAIL(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class EDGE_ENHANCE(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class EDGE_ENHANCE_MORE(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class EMBOSS(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class FIND_EDGES(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class SHARPEN(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class SMOOTH(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class SMOOTH_MORE(BuiltinFilter):
    name: str
    filterargs: _FilterArgs

class Color3DLUT(MultibandFilter):
    name: str
    size: list[int]
    channels: int
    mode: str | None
    table: Incomplete
    def __init__(self, size: int | Iterable[int], table, channels: int = 3, target_mode: str | None = None, **kwargs) -> None: ...
    @classmethod
    def generate(
        cls,
        size: int | tuple[int, int, int],
        callback: Callable[[float, float, float], Iterable[float]],
        channels: int = 3,
        target_mode: str | None = None,
    ) -> Self: ...
    def transform(
        self,
        callback: Callable[..., Iterable[float]],
        with_normals: bool = False,
        channels: Literal[3, 4] | None = None,
        target_mode: Incomplete | None = None,
    ) -> Self: ...
    def filter(self, image) -> Image: ...



from _typeshed import SupportsGetItem, Unused
from collections.abc import Generator, Iterable, Iterator, Sized
from typing import overload

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatch

@_dispatch
def enumerate_all_cliques(G: Graph[_Node]) -> Generator[list[_Node], None, None]: ...
@_dispatch
def find_cliques(G: Graph[_Node], nodes: SupportsGetItem[slice, _Node] | None = None) -> Generator[list[_Node], None, None]: ...
@_dispatch
def find_cliques_recursive(G: Graph[_Node], nodes: SupportsGetItem[slice, _Node] | None = None) -> Iterator[list[_Node]]: ...
@_dispatch
def make_max_clique_graph(G: Graph[_Node], create_using: type[Graph[_Node]] | None = None) -> Graph[_Node]: ...
@_dispatch
def make_clique_bipartite(
    G: Graph[_Node], fpos: Unused = None, create_using: type[Graph[_Node]] | None = None, name: Unused = None
) -> Graph[_Node]: ...
@overload
def node_clique_number(  # type: ignore[misc]  # Incompatible return types
    G: Graph[_Node],
    nodes: Iterable[_Node] | None = None,
    cliques: Iterable[Iterable[_Node]] | None = None,
    separate_nodes: Unused = False,
) -> dict[_Node, int]: ...
@overload
def node_clique_number(
    G: Graph[_Node], nodes: _Node, cliques: Iterable[Sized] | None = None, separate_nodes: Unused = False
) -> int: ...



from _typeshed import Incomplete
from typing import Any

from google.cloud.ndb import model

class _BaseMetadata(model.Model):
    KIND_NAME: str
    def __new__(cls, *args, **kwargs): ...

class Namespace(_BaseMetadata):
    KIND_NAME: str
    EMPTY_NAMESPACE_ID: int
    @property
    def namespace_name(self): ...
    @classmethod
    def key_for_namespace(cls, namespace): ...
    @classmethod
    def key_to_namespace(cls, key): ...

class Kind(_BaseMetadata):
    KIND_NAME: str
    @property
    def kind_name(self): ...
    @classmethod
    def key_for_kind(cls, kind): ...
    @classmethod
    def key_to_kind(cls, key): ...

class Property(_BaseMetadata):
    KIND_NAME: str
    @property
    def property_name(self): ...
    @property
    def kind_name(self): ...
    property_representation: Any
    @classmethod
    def key_for_kind(cls, kind): ...
    @classmethod
    def key_for_property(cls, kind, property): ...
    @classmethod
    def key_to_kind(cls, key): ...
    @classmethod
    def key_to_property(cls, key): ...

class EntityGroup:
    def __new__(cls, *args, **kwargs): ...

def get_entity_group_version(*args, **kwargs) -> None: ...
def get_kinds(start: Incomplete | None = ..., end: Incomplete | None = ...): ...
def get_namespaces(start: Incomplete | None = ..., end: Incomplete | None = ...): ...
def get_properties_of_kind(kind, start: Incomplete | None = ..., end: Incomplete | None = ...): ...
def get_representations_of_kind(kind, start: Incomplete | None = ..., end: Incomplete | None = ...): ...



from _typeshed import ConvertibleToInt, Incomplete, SupportsIter
from collections.abc import Iterator
from typing import Any, ClassVar, Final, Protocol
from typing_extensions import Self

from openpyxl.descriptors import MetaSerialisable
from openpyxl.xml.functions import Element

from ..xml._functions_overloads import _HasAttrib, _HasGet, _HasTagAndGet, _HasText, _SupportsFindChartLines

# For any override directly re-using Serialisable.from_tree
class _ChildSerialisableTreeElement(_HasAttrib, _HasText, SupportsIter[Incomplete], Protocol): ...
class _SerialisableTreeElement(_HasGet[object], _SupportsFindChartLines, _ChildSerialisableTreeElement, Protocol): ...

KEYWORDS: Final[frozenset[str]]
seq_types: Final[tuple[type[list[Any]], type[tuple[Any, ...]]]]

class Serialisable(metaclass=MetaSerialisable):
    # These dunders are always set at runtime by MetaSerialisable so they can't be None
    __attrs__: ClassVar[tuple[str, ...]]
    __nested__: ClassVar[tuple[str, ...]]
    __elements__: ClassVar[tuple[str, ...]]
    __namespaced__: ClassVar[tuple[tuple[str, str], ...]]
    idx_base: int
    # Needs overrides in many sub-classes. But a lot of subclasses are instantiated without overriding it, so can't be abstract
    # Subclasses "overrides" this property with a ClassVar, and Serialisable is too widely used,
    # so it can't be typed as NoReturn either without introducing many false-positives.
    @property
    def tagname(self) -> str: ...
    namespace: ClassVar[str | None]
    # Note: To respect the Liskov substitution principle, the protocol for node includes all child class requirements.
    # Same with the return type to avoid override issues.
    # See comment in xml/functions.pyi as to why use a protocol instead of Element
    # Child classes should be more precise than _SerialisableTreeElement !
    # Use _ChildSerialisableTreeElement instead for child classes that reuse Serialisable.from_tree directly.
    @classmethod
    def from_tree(cls, node: _SerialisableTreeElement) -> Self | None: ...
    # Note: To respect the Liskov substitution principle, idx is a type union of all child class requirements.
    # Use Unused instead for child classes that reuse Serialisable.to_tree directly.
    def to_tree(
        self,
        tagname: str | None = None,
        idx: _HasTagAndGet[ConvertibleToInt] | ConvertibleToInt | None = None,
        namespace: str | None = None,
    ) -> Element: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __add__(self, other): ...
    def __copy__(self): ...



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Bool, NoneSet, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.geometry import GroupTransform2D, Scene3D
from openpyxl.drawing.text import Hyperlink

_GroupShapePropertiesBwMode: TypeAlias = Literal[
    "clr", "auto", "gray", "ltGray", "invGray", "grayWhite", "blackGray", "blackWhite", "black", "white", "hidden"
]

class GroupShapeProperties(Serialisable):
    tagname: ClassVar[str]
    bwMode: NoneSet[_GroupShapePropertiesBwMode]
    xfrm: Typed[GroupTransform2D, Literal[True]]
    scene3d: Typed[Scene3D, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    def __init__(
        self,
        bwMode: _GroupShapePropertiesBwMode | Literal["none"] | None = None,
        xfrm: GroupTransform2D | None = None,
        scene3d: Scene3D | None = None,
        extLst: ExtensionList | None = None,
    ) -> None: ...

class GroupLocking(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    noGrp: Bool[Literal[True]]
    noUngrp: Bool[Literal[True]]
    noSelect: Bool[Literal[True]]
    noRot: Bool[Literal[True]]
    noChangeAspect: Bool[Literal[True]]
    noMove: Bool[Literal[True]]
    noResize: Bool[Literal[True]]
    noChangeArrowheads: Bool[Literal[True]]
    noEditPoints: Bool[Literal[True]]
    noAdjustHandles: Bool[Literal[True]]
    noChangeShapeType: Bool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        noGrp: _ConvertibleToBool | None = None,
        noUngrp: _ConvertibleToBool | None = None,
        noSelect: _ConvertibleToBool | None = None,
        noRot: _ConvertibleToBool | None = None,
        noChangeAspect: _ConvertibleToBool | None = None,
        noChangeArrowheads: _ConvertibleToBool | None = None,
        noMove: _ConvertibleToBool | None = None,
        noResize: _ConvertibleToBool | None = None,
        noEditPoints: _ConvertibleToBool | None = None,
        noAdjustHandles: _ConvertibleToBool | None = None,
        noChangeShapeType: _ConvertibleToBool | None = None,
        extLst: Unused = None,
    ) -> None: ...

class NonVisualGroupDrawingShapeProps(Serialisable):
    tagname: ClassVar[str]
    grpSpLocks: Typed[GroupLocking, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, grpSpLocks: Incomplete | None = None, extLst: Unused = None) -> None: ...

class NonVisualDrawingShapeProps(Serialisable):
    tagname: ClassVar[str]
    spLocks: Typed[GroupLocking, Literal[True]]
    txBax: Bool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    txBox: Incomplete
    def __init__(
        self, spLocks: Incomplete | None = None, txBox: _ConvertibleToBool | None = None, extLst: Unused = None
    ) -> None: ...

class NonVisualDrawingProps(Serialisable):
    tagname: ClassVar[str]
    id: Incomplete
    name: String[Literal[False]]
    descr: String[Literal[True]]
    hidden: Bool[Literal[True]]
    title: String[Literal[True]]
    hlinkClick: Typed[Hyperlink, Literal[True]]
    hlinkHover: Typed[Hyperlink, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    # Source incorrectly uses a list here instead of a tuple
    __elements__: ClassVar[list[str]]  # type: ignore[assignment]
    @overload
    def __init__(
        self,
        id: Incomplete | None = None,
        *,
        name: str,
        descr: str | None = None,
        hidden: _ConvertibleToBool | None = None,
        title: str | None = None,
        hlinkClick: Hyperlink | None = None,
        hlinkHover: Hyperlink | None = None,
        extLst: ExtensionList | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        id: Incomplete | None,
        name: str,
        descr: str | None = None,
        hidden: _ConvertibleToBool | None = None,
        title: str | None = None,
        hlinkClick: Hyperlink | None = None,
        hlinkHover: Hyperlink | None = None,
        extLst: ExtensionList | None = None,
    ) -> None: ...

class NonVisualGroupShape(Serialisable):
    tagname: ClassVar[str]
    cNvPr: Typed[NonVisualDrawingProps, Literal[False]]
    cNvGrpSpPr: Typed[NonVisualGroupDrawingShapeProps, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, cNvPr: NonVisualDrawingProps, cNvGrpSpPr: NonVisualGroupDrawingShapeProps) -> None: ...



from _typeshed import Incomplete

from influxdb_client.domain.expression import Expression

class BinaryExpression(Expression):
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        type: Incomplete | None = None,
        operator: Incomplete | None = None,
        left: Incomplete | None = None,
        right: Incomplete | None = None,
    ) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type) -> None: ...
    @property
    def operator(self): ...
    @operator.setter
    def operator(self, operator) -> None: ...
    @property
    def left(self): ...
    @left.setter
    def left(self, left) -> None: ...
    @property
    def right(self): ...
    @right.setter
    def right(self, right) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import SupportsItems
from collections.abc import Callable, Iterable, Iterator
from typing import Any, Literal, NamedTuple, TypeVar, overload
from typing_extensions import Self, TypeAlias

_T = TypeVar("_T")
_ListOrTuple: TypeAlias = list[_T] | tuple[_T, ...]
_ParsedAccept: TypeAlias = tuple[str, float, list[tuple[str, str]], list[str | tuple[str, str]]]

class AcceptOffer(NamedTuple):
    type: str
    subtype: str
    params: tuple[tuple[str, str], ...]

class Accept:
    @classmethod
    def parse(cls, value: str) -> Iterator[_ParsedAccept]: ...
    @classmethod
    def parse_offer(cls, offer: str | AcceptOffer) -> AcceptOffer: ...

class AcceptValidHeader(Accept):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[_ParsedAccept] | None: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: str | None) -> Self: ...
    @overload
    def __add__(self, other: AcceptNoHeader | AcceptValidHeader | AcceptInvalidHeader) -> Self: ...
    @overload
    def __add__(self, other: SupportsItems[str, float | tuple[float, str]]) -> Self: ...
    @overload
    def __add__(self, other: _ListOrTuple[str | tuple[str, float, str] | list[Any]]) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __radd__(self, other: str | None) -> Self: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float | tuple[float, str]]) -> Self: ...
    @overload
    def __radd__(self, other: _ListOrTuple[str | tuple[str, float, str] | list[Any]]) -> Self: ...
    def accept_html(self) -> bool: ...
    @property
    def accepts_html(self) -> bool: ...
    def acceptable_offers(self, offers: Iterable[str | AcceptOffer]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptInvalidOrNoHeader(Accept):
    @property
    def parsed(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: None) -> AcceptNoHeader: ...
    @overload
    def __add__(self, other: str) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __add__(self, other: AcceptValidHeader) -> AcceptValidHeader: ...
    @overload
    def __add__(self, other: AcceptNoHeader | AcceptInvalidHeader) -> AcceptNoHeader: ...
    @overload
    def __add__(self, other: SupportsItems[str, float | tuple[float, str]]) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __add__(self, other: _ListOrTuple[str | tuple[str, float, str] | list[Any]]) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __radd__(self, other: None) -> AcceptNoHeader: ...
    @overload
    def __radd__(self, other: str) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __radd__(self, other: AcceptValidHeader) -> AcceptValidHeader: ...
    @overload
    def __radd__(self, other: AcceptNoHeader | AcceptInvalidHeader) -> AcceptNoHeader: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float | tuple[float, str]]) -> AcceptValidHeader | AcceptNoHeader: ...
    @overload
    def __radd__(self, other: _ListOrTuple[str | tuple[str, float, str] | list[Any]]) -> AcceptValidHeader | AcceptNoHeader: ...
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    def accept_html(self) -> bool: ...
    @property
    def accepts_html(self) -> bool: ...
    def acceptable_offers(self, offers: Iterable[str | AcceptOffer]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str | tuple[str, float] | list[Any]], default_match: str) -> str: ...
    def quality(self, offer: str) -> float: ...

class AcceptNoHeader(_AcceptInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    def __init__(self) -> None: ...

class AcceptInvalidHeader(_AcceptInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    def __init__(self, header_value: str) -> None: ...

@overload
def create_accept_header(header_value: None) -> AcceptNoHeader: ...
@overload
def create_accept_header(header_value: str) -> AcceptValidHeader | AcceptInvalidHeader: ...

class _AcceptProperty:
    @overload
    def __get__(self, obj: None, type: type | None = ..., /) -> property: ...
    @overload
    def __get__(self, obj: Any, type: type | None = ..., /) -> AcceptNoHeader | AcceptValidHeader | AcceptInvalidHeader: ...
    @overload
    def __set__(self, obj: Any, value: str | None, /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: AcceptNoHeader | AcceptValidHeader | AcceptInvalidHeader, /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: SupportsItems[str, float | tuple[float, str]], /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: _ListOrTuple[str | tuple[str, float, str] | list[Any]], /) -> None: ...
    def __delete__(self, obj: Any, /) -> None: ...

def accept_property() -> _AcceptProperty: ...

class AcceptCharset:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptCharsetValidHeader(AcceptCharset):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: str | None) -> Self: ...
    @overload
    def __add__(self, other: AcceptCharsetValidHeader | AcceptCharsetNoHeader | AcceptCharsetInvalidHeader) -> Self: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __add__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __radd__(self, other: str | None) -> Self: ...
    @overload
    def __radd__(self, other: AcceptCharsetValidHeader | AcceptCharsetNoHeader | AcceptCharsetInvalidHeader) -> Self: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __radd__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptCharsetInvalidOrNoHeader(AcceptCharset):
    @property
    def parsed(self) -> None: ...
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __add__(self, other: None) -> AcceptCharsetNoHeader: ...
    @overload
    def __add__(self, other: str) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __add__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __add__(self, other: AcceptCharsetNoHeader | AcceptCharsetInvalidHeader) -> AcceptCharsetNoHeader: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __add__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __radd__(self, other: None) -> AcceptCharsetNoHeader: ...
    @overload
    def __radd__(self, other: str) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __radd__(self, other: AcceptCharsetValidHeader) -> AcceptCharsetValidHeader: ...
    @overload
    def __radd__(self, other: AcceptCharsetNoHeader | AcceptCharsetInvalidHeader) -> AcceptCharsetNoHeader: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    @overload
    def __radd__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptCharsetValidHeader | AcceptCharsetNoHeader: ...
    def copy(self) -> Self: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptCharsetNoHeader(_AcceptCharsetInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    def __init__(self) -> None: ...

class AcceptCharsetInvalidHeader(_AcceptCharsetInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    def __init__(self, header_value: str) -> None: ...

@overload
def create_accept_charset_header(header_value: None) -> AcceptCharsetNoHeader: ...
@overload
def create_accept_charset_header(header_value: str) -> AcceptCharsetValidHeader | AcceptCharsetInvalidHeader: ...

class _AcceptCharsetProperty:
    @overload
    def __get__(self, obj: None, type: type | None = ..., /) -> property: ...
    @overload
    def __get__(
        self, obj: Any, type: type | None = ..., /
    ) -> AcceptCharsetNoHeader | AcceptCharsetValidHeader | AcceptCharsetInvalidHeader: ...
    @overload
    def __set__(self, obj: Any, value: str | None, /) -> None: ...
    @overload
    def __set__(
        self, obj: Any, value: AcceptCharsetNoHeader | AcceptCharsetValidHeader | AcceptCharsetInvalidHeader, /
    ) -> None: ...
    @overload
    def __set__(self, obj: Any, value: SupportsItems[str, float], /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: _ListOrTuple[str | tuple[str, float] | list[Any]], /) -> None: ...
    def __delete__(self, obj: Any, /) -> None: ...

def accept_charset_property() -> _AcceptCharsetProperty: ...

class AcceptEncoding:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptEncodingValidHeader(AcceptEncoding):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: str | None) -> Self: ...
    @overload
    def __add__(self, other: AcceptEncodingValidHeader | AcceptEncodingNoHeader | AcceptEncodingInvalidHeader) -> Self: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __add__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __radd__(self, other: str | None) -> Self: ...
    @overload
    def __radd__(self, other: AcceptEncodingValidHeader | AcceptEncodingNoHeader | AcceptEncodingInvalidHeader) -> Self: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __radd__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptEncodingInvalidOrNoHeader(AcceptEncoding):
    @property
    def parsed(self) -> None: ...
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __add__(self, other: None) -> AcceptEncodingNoHeader: ...
    @overload
    def __add__(self, other: str) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __add__(self, other: AcceptEncodingValidHeader) -> AcceptEncodingValidHeader: ...
    @overload
    def __add__(self, other: AcceptEncodingNoHeader | AcceptEncodingInvalidHeader) -> AcceptEncodingNoHeader: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __add__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __radd__(self, other: None) -> AcceptEncodingNoHeader: ...
    @overload
    def __radd__(self, other: str) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingValidHeader) -> AcceptEncodingValidHeader: ...
    @overload
    def __radd__(self, other: AcceptEncodingNoHeader | AcceptEncodingInvalidHeader) -> AcceptEncodingNoHeader: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    @overload
    def __radd__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptEncodingValidHeader | AcceptEncodingNoHeader: ...
    def copy(self) -> Self: ...
    def acceptable_offers(self, offers: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptEncodingNoHeader(_AcceptEncodingInvalidOrNoHeader):
    @property
    def header_value(self) -> None: ...
    def __init__(self) -> None: ...

class AcceptEncodingInvalidHeader(_AcceptEncodingInvalidOrNoHeader):
    @property
    def header_value(self) -> str: ...
    def __init__(self, header_value: str) -> None: ...

@overload
def create_accept_encoding_header(header_value: None) -> AcceptEncodingNoHeader: ...
@overload
def create_accept_encoding_header(header_value: str) -> AcceptEncodingValidHeader | AcceptEncodingInvalidHeader: ...

class _AcceptEncodingProperty:
    @overload
    def __get__(self, obj: None, type: type | None = ..., /) -> property: ...
    @overload
    def __get__(
        self, obj: Any, type: type | None = ..., /
    ) -> AcceptEncodingNoHeader | AcceptEncodingValidHeader | AcceptEncodingInvalidHeader: ...
    @overload
    def __set__(self, obj: Any, value: str | None, /) -> None: ...
    @overload
    def __set__(
        self, obj: Any, value: AcceptEncodingNoHeader | AcceptEncodingValidHeader | AcceptEncodingInvalidHeader, /
    ) -> None: ...
    @overload
    def __set__(self, obj: Any, value: SupportsItems[str, float], /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: _ListOrTuple[str | tuple[str, float] | list[Any]], /) -> None: ...
    def __delete__(self, obj: Any, /) -> None: ...

def accept_encoding_property() -> _AcceptEncodingProperty: ...

class AcceptLanguage:
    @classmethod
    def parse(cls, value: str) -> Iterator[tuple[str, float]]: ...

class AcceptLanguageValidHeader(AcceptLanguage):
    @property
    def header_value(self) -> str: ...
    @property
    def parsed(self) -> list[tuple[str, float]]: ...
    def __init__(self, header_value: str) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def __add__(self, other: str | None) -> Self: ...
    @overload
    def __add__(self, other: AcceptLanguageValidHeader | AcceptLanguageNoHeader | AcceptLanguageInvalidHeader) -> Self: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __add__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def __bool__(self) -> Literal[True]: ...
    def __contains__(self, offer: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __radd__(self, other: str | None) -> Self: ...
    @overload
    def __radd__(self, other: AcceptLanguageValidHeader | AcceptLanguageNoHeader | AcceptLanguageInvalidHeader) -> Self: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> Self: ...
    @overload
    def __radd__(self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]) -> Self: ...
    def basic_filtering(self, language_tags: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    @overload
    def lookup(
        self,
        language_tags: Iterable[str],
        default_range: str | None = None,
        default_tag: str = ...,
        default: str | Callable[[], str | None] | None = None,
    ) -> str | None: ...
    @overload
    def lookup(
        self,
        language_tags: Iterable[str],
        default_range: str | None = None,
        default_tag: str | None = None,
        default: str | Callable[[], str | None] = ...,
    ) -> str | None: ...
    def quality(self, offer: str) -> float | None: ...

class _AcceptLanguageInvalidOrNoHeader(AcceptLanguage):
    @property
    def parsed(self) -> None: ...
    def __bool__(self) -> Literal[False]: ...
    def __contains__(self, offer: str) -> Literal[True]: ...
    def __iter__(self) -> Iterator[str]: ...
    @overload
    def __add__(self, other: None) -> AcceptLanguageNoHeader: ...
    @overload
    def __add__(self, other: str) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __add__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __add__(self, other: AcceptLanguageNoHeader | AcceptLanguageInvalidHeader) -> AcceptLanguageNoHeader: ...
    @overload
    def __add__(self, other: SupportsItems[str, float]) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __add__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __radd__(self, other: None) -> AcceptLanguageNoHeader: ...
    @overload
    def __radd__(self, other: str) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __radd__(self, other: AcceptLanguageValidHeader) -> AcceptLanguageValidHeader: ...
    @overload
    def __radd__(self, other: AcceptLanguageNoHeader | AcceptLanguageInvalidHeader) -> AcceptLanguageNoHeader: ...
    @overload
    def __radd__(self, other: SupportsItems[str, float]) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    @overload
    def __radd__(
        self, other: _ListOrTuple[str | tuple[str, float] | list[Any]]
    ) -> AcceptLanguageValidHeader | AcceptLanguageNoHeader: ...
    def copy(self) -> Self: ...
    def basic_filtering(self, language_tags: Iterable[str]) -> list[tuple[str, float]]: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: None = None) -> str | None: ...
    @overload
    def best_match(self, offers: Iterable[str], default_match: str) -> str: ...
    @overload
    def lookup(
        self,
        language_tags: Iterable[str],
        default_range: str | None = None,
        default_tag: str = ...,
        default: str | Callable[[], str | None] | None = None,
    ) -> str | None: ...
    @overload
    def lookup(
        self,
        language_tags: Iterable[str],
        default_range: str | None = None,
        default_tag: str | None = None,
        default: str | Callable[[], str | None] = ...,
    ) -> str | None: ...
    def quality(self, offer: str) -> float | None: ...

class AcceptLanguageNoHeader(_AcceptLanguageInvalidOrNoHeader):
    def __init__(self) -> None: ...
    @property
    def header_value(self) -> None: ...

class AcceptLanguageInvalidHeader(_AcceptLanguageInvalidOrNoHeader):
    def __init__(self, header_value: str) -> None: ...
    @property
    def header_value(self) -> str: ...

@overload
def create_accept_language_header(header_value: None) -> AcceptLanguageNoHeader: ...
@overload
def create_accept_language_header(header_value: str) -> AcceptLanguageValidHeader | AcceptLanguageInvalidHeader: ...

class _AcceptLanguageProperty:
    @overload
    def __get__(self, obj: None, type: type | None = ..., /) -> property: ...
    @overload
    def __get__(
        self, obj: Any, type: type | None = ..., /
    ) -> AcceptLanguageNoHeader | AcceptLanguageValidHeader | AcceptLanguageInvalidHeader: ...
    @overload
    def __set__(self, obj: Any, value: str | None, /) -> None: ...
    @overload
    def __set__(
        self, obj: Any, value: AcceptLanguageNoHeader | AcceptLanguageValidHeader | AcceptLanguageInvalidHeader, /
    ) -> None: ...
    @overload
    def __set__(self, obj: Any, value: SupportsItems[str, float], /) -> None: ...
    @overload
    def __set__(self, obj: Any, value: _ListOrTuple[str | tuple[str, float] | list[Any]], /) -> None: ...
    def __delete__(self, obj: Any, /) -> None: ...

def accept_language_property() -> _AcceptLanguageProperty: ...



from _typeshed import Incomplete

unicode = str
long = int
xrange = range
bytechr: Incomplete

class CTypesType(type): ...

class CTypesData:
    __metaclass__: Incomplete
    __name__: str
    def __init__(self, *args) -> None: ...
    def __iter__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __hash__(self) -> int: ...
    def __repr__(self, c_name: str | None = None): ...

class CTypesGenericPrimitive(CTypesData):
    def __hash__(self) -> int: ...

class CTypesGenericArray(CTypesData):
    def __iter__(self): ...

class CTypesGenericPtr(CTypesData):
    kind: str
    def __nonzero__(self) -> bool: ...
    def __bool__(self) -> bool: ...

class CTypesBaseStructOrUnion(CTypesData): ...

class CTypesBackend:
    PRIMITIVE_TYPES: Incomplete
    RTLD_LAZY: int
    RTLD_NOW: int
    RTLD_GLOBAL: Incomplete
    RTLD_LOCAL: Incomplete
    def __init__(self) -> None: ...
    ffi: Incomplete
    def set_ffi(self, ffi) -> None: ...
    def load_library(self, path, flags: int = 0): ...
    def new_void_type(self): ...
    def new_primitive_type(self, name): ...
    def new_pointer_type(self, BItem): ...
    def new_array_type(self, CTypesPtr, length): ...
    def new_struct_type(self, name): ...
    def new_union_type(self, name): ...
    def complete_struct_or_union(
        self, CTypesStructOrUnion, fields, tp, totalsize: int = -1, totalalignment: int = -1, sflags: int = 0, pack: int = 0
    ): ...
    def new_function_type(self, BArgs, BResult, has_varargs): ...
    def new_enum_type(self, name, enumerators, enumvalues, CTypesInt): ...
    def get_errno(self): ...
    def set_errno(self, value) -> None: ...
    def string(self, b, maxlen: int = -1): ...
    def buffer(self, bptr, size: int = -1) -> None: ...
    def sizeof(self, cdata_or_BType) -> int: ...
    def alignof(self, BType) -> int: ...
    def newp(self, BType, source): ...
    def cast(self, BType, source): ...
    def callback(self, BType, source, error, onerror): ...
    def gcp(self, cdata, destructor, size: int = 0): ...
    typeof: Incomplete
    def getcname(self, BType, replace_with): ...
    def typeoffsetof(self, BType, fieldname, num: int = 0): ...
    def rawaddressof(self, BTypePtr, cdata, offset: Incomplete | None = None): ...

class CTypesLibrary:
    backend: Incomplete
    cdll: Incomplete
    def __init__(self, backend, cdll) -> None: ...
    def load_function(self, BType, name): ...
    def read_variable(self, BType, name): ...
    def write_variable(self, BType, name, value) -> None: ...



from . import tempfile as tempfile
from .threadpool import (
    open as open,
    stderr as stderr,
    stderr_bytes as stderr_bytes,
    stdin as stdin,
    stdin_bytes as stdin_bytes,
    stdout as stdout,
    stdout_bytes as stdout_bytes,
)
